@@ -41,17 +41,15 @@ class LegacySchemaAliasingTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
     def _assert_sql(self, element, legacy_sql, modern_sql=None):
+        dialect = mssql.dialect(legacy_schema_aliasing=True)
+
+        self.assert_compile(
+            element,
+            legacy_sql,
+            dialect=dialect
+        )
+
         dialect = mssql.dialect()
-
-        with assertions.expect_warnings(
-                "legacy_schema_aliasing flag is defaulted to True.*"):
-            self.assert_compile(
-                element,
-                legacy_sql,
-                dialect=dialect
-            )
-
-        dialect = mssql.dialect(legacy_schema_aliasing=False)
         self.assert_compile(
             element,
             modern_sql or "foob",
@@ -136,6 +134,7 @@ class LegacySchemaAliasingTest(fixtures.TestBase, AssertsCompiledSQL):
 class IdentityInsertTest(fixtures.TestBase, AssertsCompiledSQL):
     __only_on__ = 'mssql'
     __dialect__ = mssql.MSDialect()
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
@@ -189,6 +188,7 @@ class IdentityInsertTest(fixtures.TestBase, AssertsCompiledSQL):
 class QueryUnicodeTest(fixtures.TestBase):
 
     __only_on__ = 'mssql'
+    __backend__ = True
 
     def test_convert_unicode(self):
         meta = MetaData(testing.db)
@@ -216,6 +216,7 @@ class QueryUnicodeTest(fixtures.TestBase):
 
 class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
     __only_on__ = 'mssql'
+    __backend__ = True
 
     def test_fetchid_trigger(self):
         """
@@ -476,7 +477,7 @@ def full_text_search_missing():
             connection.execute('CREATE FULLTEXT CATALOG Catalog AS '
                                'DEFAULT')
             return False
-        except:
+        except Exception:
             return True
     finally:
         connection.close()
@@ -486,6 +487,7 @@ class MatchTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __only_on__ = 'mssql'
     __skip_if__ = full_text_search_missing,
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
