@@ -1,5 +1,6 @@
 # -*- encoding: utf-8
 from sqlalchemy.testing import eq_, engines, pickleable, assert_raises_message
+from sqlalchemy.testing import is_, is_not_
 import datetime
 import os
 from sqlalchemy import Table, Column, MetaData, Float, \
@@ -64,6 +65,18 @@ class MSDateTypeTest(fixtures.TestBase):
             result_processor, 'abc'
         )
 
+    def test_extract(self):
+        from sqlalchemy import extract
+        fivedaysago = datetime.datetime.now() \
+            - datetime.timedelta(days=5)
+        for field, exp in ('year', fivedaysago.year), \
+                ('month', fivedaysago.month), ('day', fivedaysago.day):
+            r = testing.db.execute(
+                select([
+                    extract(field, fivedaysago)])
+            ).scalar()
+            eq_(r, exp)
+
 
 class TypeDDLTest(fixtures.TestBase):
 
@@ -414,6 +427,7 @@ class TypeDDLTest(fixtures.TestBase):
                         % (col.name, columns[index][3]))
             self.assert_(repr(col))
 
+
 metadata = None
 
 
@@ -431,7 +445,7 @@ class TypeRoundTripTest(
 
     @testing.fails_on_everything_except(
         'mssql+pyodbc',
-        'this is some pyodbc-specific feature')
+        'mssql+mxodbc')
     def test_decimal_notation(self):
         numeric_table = Table(
             'numeric_table', metadata,
@@ -715,44 +729,37 @@ class TypeRoundTripTest(
     def test_autoincrement(self):
         Table(
             'ai_1', metadata,
-            Column('int_y', Integer, primary_key=True),
+            Column('int_y', Integer, primary_key=True, autoincrement=True),
             Column(
-                'int_n', Integer, DefaultClause('0'),
-                primary_key=True, autoincrement=False))
+                'int_n', Integer, DefaultClause('0'), primary_key=True))
         Table(
             'ai_2', metadata,
-            Column('int_y', Integer, primary_key=True),
-            Column('int_n', Integer, DefaultClause('0'),
-                   primary_key=True, autoincrement=False))
+            Column('int_y', Integer, primary_key=True, autoincrement=True),
+            Column('int_n', Integer, DefaultClause('0'), primary_key=True))
         Table(
             'ai_3', metadata,
-            Column('int_n', Integer, DefaultClause('0'),
-                   primary_key=True, autoincrement=False),
-            Column('int_y', Integer, primary_key=True))
+            Column('int_n', Integer, DefaultClause('0'), primary_key=True),
+            Column('int_y', Integer, primary_key=True, autoincrement=True))
 
         Table(
             'ai_4', metadata,
-            Column('int_n', Integer, DefaultClause('0'),
-                   primary_key=True, autoincrement=False),
-            Column('int_n2', Integer, DefaultClause('0'),
-                   primary_key=True, autoincrement=False))
+            Column('int_n', Integer, DefaultClause('0'), primary_key=True),
+            Column('int_n2', Integer, DefaultClause('0'), primary_key=True))
         Table(
             'ai_5', metadata,
-            Column('int_y', Integer, primary_key=True),
-            Column('int_n', Integer, DefaultClause('0'),
-                   primary_key=True, autoincrement=False))
+            Column('int_y', Integer, primary_key=True, autoincrement=True),
+            Column('int_n', Integer, DefaultClause('0'), primary_key=True))
         Table(
             'ai_6', metadata,
-            Column('o1', String(1), DefaultClause('x'),
-                   primary_key=True),
-            Column('int_y', Integer, primary_key=True))
+            Column('o1', String(1), DefaultClause('x'), primary_key=True),
+            Column('int_y', Integer, primary_key=True, autoincrement=True))
         Table(
             'ai_7', metadata,
             Column('o1', String(1), DefaultClause('x'),
                    primary_key=True),
             Column('o2', String(1), DefaultClause('x'),
                    primary_key=True),
-            Column('int_y', Integer, primary_key=True))
+            Column('int_y', Integer, autoincrement=True, primary_key=True))
         Table(
             'ai_8', metadata,
             Column('o1', String(1), DefaultClause('x'),
@@ -768,13 +775,15 @@ class TypeRoundTripTest(
         for name in table_names:
             tbl = Table(name, mr, autoload=True)
             tbl = metadata.tables[name]
-            for c in tbl.c:
-                if c.name.startswith('int_y'):
-                    assert c.autoincrement, name
-                    assert tbl._autoincrement_column is c, name
-                elif c.name.startswith('int_n'):
-                    assert not c.autoincrement, name
-                    assert tbl._autoincrement_column is not c, name
+
+            # test that the flag itself reflects appropriately
+            for col in tbl.c:
+                if 'int_y' in col.name:
+                    is_(col.autoincrement, True)
+                    is_(tbl._autoincrement_column, col)
+                else:
+                    eq_(col.autoincrement, 'auto')
+                    is_not_(tbl._autoincrement_column, col)
 
             # mxodbc can't handle scope_identity() with DEFAULT VALUES
 
@@ -818,6 +827,7 @@ class MonkeyPatchedBinaryTest(fixtures.TestBase):
         result = module.Binary(input)
         eq_(result, expected_result)
 
+
 binary_table = None
 MyPickleType = None
 
@@ -922,17 +932,17 @@ class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
                         mypickle=MyPickleType),
                     bind=testing.db):
             with engine.connect() as conn:
-                l = conn.execute(stmt).fetchall()
-            eq_(list(stream1), list(l[0]['data']))
+                result = conn.execute(stmt).fetchall()
+            eq_(list(stream1), list(result[0]['data']))
             paddedstream = list(stream1[0:100])
             paddedstream.extend(['\x00'] * (100 - len(paddedstream)))
-            eq_(paddedstream, list(l[0]['data_slice']))
-            eq_(list(stream2), list(l[1]['data']))
-            eq_(list(stream2), list(l[1]['data_image']))
-            eq_(testobj1, l[0]['pickled'])
-            eq_(testobj2, l[1]['pickled'])
-            eq_(testobj3.moredata, l[0]['mypickle'].moredata)
-            eq_(l[0]['mypickle'].stuff, 'this is the right stuff')
+            eq_(paddedstream, list(result[0]['data_slice']))
+            eq_(list(stream2), list(result[1]['data']))
+            eq_(list(stream2), list(result[1]['data_image']))
+            eq_(testobj1, result[0]['pickled'])
+            eq_(testobj2, result[1]['pickled'])
+            eq_(testobj3.moredata, result[0]['mypickle'].moredata)
+            eq_(result[0]['mypickle'].stuff, 'this is the right stuff')
 
     def _test_binary_none(self, deprecate_large_types):
         engine = engines.testing_engine(
