@@ -9,6 +9,7 @@ from sqlalchemy import event
 from sqlalchemy import sql
 from sqlalchemy import inspect
 from sqlalchemy.dialects.mysql import base as mysql
+from sqlalchemy.dialects.mysql import reflection as _reflection
 from sqlalchemy.testing import fixtures, AssertsExecutionResults
 from sqlalchemy import testing
 
@@ -183,7 +184,7 @@ class TypeReflectionTest(fixtures.TestBase):
     def test_legacy_enum_types(self):
 
         specs = [
-            (mysql.ENUM("''","'fleem'"), mysql.ENUM("''","'fleem'")),  # noqa
+            (mysql.ENUM("''", "'fleem'"), mysql.ENUM("''", "'fleem'")),
         ]
 
         self._run_test(specs, ['enums'])
@@ -325,49 +326,55 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         meta = MetaData(testing.db)
         try:
             Table('ai_1', meta,
-                  Column('int_y', Integer, primary_key=True),
+                  Column('int_y', Integer, primary_key=True,
+                         autoincrement=True),
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True),
-                         mysql_engine='MyISAM')
+                  mysql_engine='MyISAM')
             Table('ai_2', meta,
-                  Column('int_y', Integer, primary_key=True),
+                  Column('int_y', Integer, primary_key=True,
+                         autoincrement=True),
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True),
-                         mysql_engine='MyISAM')
+                  mysql_engine='MyISAM')
             Table('ai_3', meta,
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True, autoincrement=False),
-                  Column('int_y', Integer, primary_key=True),
-                         mysql_engine='MyISAM')
+                  Column('int_y', Integer, primary_key=True,
+                         autoincrement=True),
+                  mysql_engine='MyISAM')
             Table('ai_4', meta,
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True, autoincrement=False),
                   Column('int_n2', Integer, DefaultClause('0'),
                          primary_key=True, autoincrement=False),
-                         mysql_engine='MyISAM')
+                  mysql_engine='MyISAM')
             Table('ai_5', meta,
-                  Column('int_y', Integer, primary_key=True),
+                  Column('int_y', Integer, primary_key=True,
+                         autoincrement=True),
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True, autoincrement=False),
-                         mysql_engine='MyISAM')
+                  mysql_engine='MyISAM')
             Table('ai_6', meta,
                   Column('o1', String(1), DefaultClause('x'),
                          primary_key=True),
-                  Column('int_y', Integer, primary_key=True),
-                         mysql_engine='MyISAM')
+                  Column('int_y', Integer, primary_key=True,
+                         autoincrement=True),
+                  mysql_engine='MyISAM')
             Table('ai_7', meta,
                   Column('o1', String(1), DefaultClause('x'),
                          primary_key=True),
                   Column('o2', String(1), DefaultClause('x'),
                          primary_key=True),
-                  Column('int_y', Integer, primary_key=True),
-                         mysql_engine='MyISAM')
+                  Column('int_y', Integer, primary_key=True,
+                         autoincrement=True),
+                  mysql_engine='MyISAM')
             Table('ai_8', meta,
                   Column('o1', String(1), DefaultClause('x'),
                          primary_key=True),
                   Column('o2', String(1), DefaultClause('x'),
                          primary_key=True),
-                         mysql_engine='MyISAM')
+                  mysql_engine='MyISAM')
             meta.create_all()
 
             table_names = ['ai_1', 'ai_2', 'ai_3', 'ai_4',
@@ -392,7 +399,10 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
 
     @testing.provide_metadata
     def test_view_reflection(self):
-        Table('x', self.metadata, Column('a', Integer), Column('b', String(50)))
+        Table('x',
+              self.metadata,
+              Column('a', Integer),
+              Column('b', String(50)))
         self.metadata.create_all()
 
         with testing.db.connect() as conn:
@@ -420,7 +430,6 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                 [('a', mysql.INTEGER), ('b', mysql.VARCHAR)]
             )
 
-
     @testing.exclude('mysql', '<', (5, 0, 0), 'no information_schema support')
     def test_system_views(self):
         dialect = testing.db.dialect
@@ -525,7 +534,7 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
 class RawReflectionTest(fixtures.TestBase):
     def setup(self):
         dialect = mysql.dialect()
-        self.parser = mysql.MySQLTableDefinitionParser(
+        self.parser = _reflection.MySQLTableDefinitionParser(
             dialect, dialect.identifier_preparer)
 
     def test_key_reflection(self):
@@ -545,6 +554,18 @@ class RawReflectionTest(fixtures.TestBase):
             '  PRIMARY KEY (`id`) USING BTREE KEY_BLOCK_SIZE  = 16')
         assert not regex.match(
             '  PRIMARY KEY (`id`) USING BTREE KEY_BLOCK_SIZE = = 16')
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'comment'")
+        # `SHOW CREATE TABLE` returns COMMENT '''comment'
+        # after creating table with COMMENT '\'comment'
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT '''comment'")
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'comment'''")
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'prefix''suffix'")
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'prefix''text''suffix'")
 
     def test_fk_reflection(self):
         regex = self.parser._re_constraint
@@ -554,14 +575,11 @@ class RawReflectionTest(fixtures.TestBase):
                         'REFERENCES `users` (`id`) '
                         'ON DELETE CASCADE ON UPDATE CASCADE')
         eq_(m.groups(), ('addresses_user_id_fkey', '`user_id`',
-                            '`users`', '`id`', None, 'CASCADE', 'CASCADE'))
-
+                         '`users`', '`id`', None, 'CASCADE', 'CASCADE'))
 
         m = regex.match('  CONSTRAINT `addresses_user_id_fkey` '
                         'FOREIGN KEY (`user_id`) '
                         'REFERENCES `users` (`id`) '
                         'ON DELETE CASCADE ON UPDATE SET NULL')
         eq_(m.groups(), ('addresses_user_id_fkey', '`user_id`',
-                            '`users`', '`id`', None, 'CASCADE', 'SET NULL'))
-
-
+                         '`users`', '`id`', None, 'CASCADE', 'SET NULL'))
