@@ -1,18 +1,23 @@
 # coding: utf-8
 
-from sqlalchemy.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing import eq_, assert_raises, assert_raises_message, is_
 from sqlalchemy import *
 from sqlalchemy import sql, exc, schema
 from sqlalchemy.util import u
 from sqlalchemy import util
 from sqlalchemy.dialects.mysql import base as mysql
-from sqlalchemy.testing import fixtures, AssertsCompiledSQL, AssertsExecutionResults
+from sqlalchemy.testing import (fixtures,
+                                AssertsCompiledSQL,
+                                AssertsExecutionResults)
 from sqlalchemy import testing
 import datetime
 import decimal
+from sqlalchemy import types as sqltypes
 
 
-class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
+class TypesTest(fixtures.TestBase,
+                AssertsExecutionResults,
+                AssertsCompiledSQL):
     "Test MySQL column types"
 
     __dialect__ = mysql.dialect()
@@ -32,11 +37,11 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
              'NUMERIC'),
             (mysql.MSNumeric, [12], {},
              'NUMERIC(12)'),
-            (mysql.MSNumeric, [12, 4], {'unsigned':True},
+            (mysql.MSNumeric, [12, 4], {'unsigned': True},
              'NUMERIC(12, 4) UNSIGNED'),
-            (mysql.MSNumeric, [12, 4], {'zerofill':True},
+            (mysql.MSNumeric, [12, 4], {'zerofill': True},
              'NUMERIC(12, 4) ZEROFILL'),
-            (mysql.MSNumeric, [12, 4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSNumeric, [12, 4], {'zerofill': True, 'unsigned': True},
              'NUMERIC(12, 4) UNSIGNED ZEROFILL'),
 
             (mysql.MSDecimal, [], {},
@@ -47,29 +52,29 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
              'DECIMAL(12)'),
             (mysql.MSDecimal, [12, None], {},
              'DECIMAL(12)'),
-            (mysql.MSDecimal, [12, 4], {'unsigned':True},
+            (mysql.MSDecimal, [12, 4], {'unsigned': True},
              'DECIMAL(12, 4) UNSIGNED'),
-            (mysql.MSDecimal, [12, 4], {'zerofill':True},
+            (mysql.MSDecimal, [12, 4], {'zerofill': True},
              'DECIMAL(12, 4) ZEROFILL'),
-            (mysql.MSDecimal, [12, 4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSDecimal, [12, 4], {'zerofill': True, 'unsigned': True},
              'DECIMAL(12, 4) UNSIGNED ZEROFILL'),
 
             (mysql.MSDouble, [None, None], {},
              'DOUBLE'),
-            (mysql.MSDouble, [12, 4], {'unsigned':True},
+            (mysql.MSDouble, [12, 4], {'unsigned': True},
              'DOUBLE(12, 4) UNSIGNED'),
-            (mysql.MSDouble, [12, 4], {'zerofill':True},
+            (mysql.MSDouble, [12, 4], {'zerofill': True},
              'DOUBLE(12, 4) ZEROFILL'),
-            (mysql.MSDouble, [12, 4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSDouble, [12, 4], {'zerofill': True, 'unsigned': True},
              'DOUBLE(12, 4) UNSIGNED ZEROFILL'),
 
             (mysql.MSReal, [None, None], {},
              'REAL'),
-            (mysql.MSReal, [12, 4], {'unsigned':True},
+            (mysql.MSReal, [12, 4], {'unsigned': True},
              'REAL(12, 4) UNSIGNED'),
-            (mysql.MSReal, [12, 4], {'zerofill':True},
+            (mysql.MSReal, [12, 4], {'zerofill': True},
              'REAL(12, 4) ZEROFILL'),
-            (mysql.MSReal, [12, 4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSReal, [12, 4], {'zerofill': True, 'unsigned': True},
              'REAL(12, 4) UNSIGNED ZEROFILL'),
 
             (mysql.MSFloat, [], {},
@@ -80,68 +85,68 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
              'FLOAT(12)'),
             (mysql.MSFloat, [12, 4], {},
              'FLOAT(12, 4)'),
-            (mysql.MSFloat, [12, 4], {'unsigned':True},
+            (mysql.MSFloat, [12, 4], {'unsigned': True},
              'FLOAT(12, 4) UNSIGNED'),
-            (mysql.MSFloat, [12, 4], {'zerofill':True},
+            (mysql.MSFloat, [12, 4], {'zerofill': True},
              'FLOAT(12, 4) ZEROFILL'),
-            (mysql.MSFloat, [12, 4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSFloat, [12, 4], {'zerofill': True, 'unsigned': True},
              'FLOAT(12, 4) UNSIGNED ZEROFILL'),
 
             (mysql.MSInteger, [], {},
              'INTEGER'),
             (mysql.MSInteger, [4], {},
              'INTEGER(4)'),
-            (mysql.MSInteger, [4], {'unsigned':True},
+            (mysql.MSInteger, [4], {'unsigned': True},
              'INTEGER(4) UNSIGNED'),
-            (mysql.MSInteger, [4], {'zerofill':True},
+            (mysql.MSInteger, [4], {'zerofill': True},
              'INTEGER(4) ZEROFILL'),
-            (mysql.MSInteger, [4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSInteger, [4], {'zerofill': True, 'unsigned': True},
              'INTEGER(4) UNSIGNED ZEROFILL'),
 
             (mysql.MSBigInteger, [], {},
              'BIGINT'),
             (mysql.MSBigInteger, [4], {},
              'BIGINT(4)'),
-            (mysql.MSBigInteger, [4], {'unsigned':True},
+            (mysql.MSBigInteger, [4], {'unsigned': True},
              'BIGINT(4) UNSIGNED'),
-            (mysql.MSBigInteger, [4], {'zerofill':True},
+            (mysql.MSBigInteger, [4], {'zerofill': True},
              'BIGINT(4) ZEROFILL'),
-            (mysql.MSBigInteger, [4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSBigInteger, [4], {'zerofill': True, 'unsigned': True},
              'BIGINT(4) UNSIGNED ZEROFILL'),
 
-             (mysql.MSMediumInteger, [], {},
-              'MEDIUMINT'),
-             (mysql.MSMediumInteger, [4], {},
-              'MEDIUMINT(4)'),
-             (mysql.MSMediumInteger, [4], {'unsigned':True},
-              'MEDIUMINT(4) UNSIGNED'),
-             (mysql.MSMediumInteger, [4], {'zerofill':True},
-              'MEDIUMINT(4) ZEROFILL'),
-             (mysql.MSMediumInteger, [4], {'zerofill':True, 'unsigned':True},
-              'MEDIUMINT(4) UNSIGNED ZEROFILL'),
+            (mysql.MSMediumInteger, [], {},
+             'MEDIUMINT'),
+            (mysql.MSMediumInteger, [4], {},
+             'MEDIUMINT(4)'),
+            (mysql.MSMediumInteger, [4], {'unsigned': True},
+             'MEDIUMINT(4) UNSIGNED'),
+            (mysql.MSMediumInteger, [4], {'zerofill': True},
+             'MEDIUMINT(4) ZEROFILL'),
+            (mysql.MSMediumInteger, [4], {'zerofill': True, 'unsigned': True},
+             'MEDIUMINT(4) UNSIGNED ZEROFILL'),
 
             (mysql.MSTinyInteger, [], {},
              'TINYINT'),
             (mysql.MSTinyInteger, [1], {},
              'TINYINT(1)'),
-            (mysql.MSTinyInteger, [1], {'unsigned':True},
+            (mysql.MSTinyInteger, [1], {'unsigned': True},
              'TINYINT(1) UNSIGNED'),
-            (mysql.MSTinyInteger, [1], {'zerofill':True},
+            (mysql.MSTinyInteger, [1], {'zerofill': True},
              'TINYINT(1) ZEROFILL'),
-            (mysql.MSTinyInteger, [1], {'zerofill':True, 'unsigned':True},
+            (mysql.MSTinyInteger, [1], {'zerofill': True, 'unsigned': True},
              'TINYINT(1) UNSIGNED ZEROFILL'),
 
             (mysql.MSSmallInteger, [], {},
              'SMALLINT'),
             (mysql.MSSmallInteger, [4], {},
              'SMALLINT(4)'),
-            (mysql.MSSmallInteger, [4], {'unsigned':True},
+            (mysql.MSSmallInteger, [4], {'unsigned': True},
              'SMALLINT(4) UNSIGNED'),
-            (mysql.MSSmallInteger, [4], {'zerofill':True},
+            (mysql.MSSmallInteger, [4], {'zerofill': True},
              'SMALLINT(4) ZEROFILL'),
-            (mysql.MSSmallInteger, [4], {'zerofill':True, 'unsigned':True},
+            (mysql.MSSmallInteger, [4], {'zerofill': True, 'unsigned': True},
              'SMALLINT(4) UNSIGNED ZEROFILL'),
-           ]
+        ]
 
         for type_, args, kw, res in columns:
             type_inst = type_(*args, **kw)
@@ -160,11 +165,10 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     @testing.provide_metadata
     def test_precision_float_roundtrip(self):
         t = Table('t', self.metadata,
-                    Column('scale_value', mysql.DOUBLE(
-                            precision=15, scale=12, asdecimal=True)),
-                    Column('unscale_value', mysql.DOUBLE(
-                            decimal_return_scale=12, asdecimal=True))
-            )
+                  Column('scale_value', mysql.DOUBLE(
+                         precision=15, scale=12, asdecimal=True)),
+                  Column('unscale_value', mysql.DOUBLE(
+                         decimal_return_scale=12, asdecimal=True)))
         t.create(testing.db)
         testing.db.execute(
             t.insert(), scale_value=45.768392065789,
@@ -183,25 +187,25 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         columns = [
             (mysql.MSChar, [1], {},
              'CHAR(1)'),
-             (mysql.NCHAR, [1], {},
-              'NATIONAL CHAR(1)'),
-            (mysql.MSChar, [1], {'binary':True},
+            (mysql.NCHAR, [1], {},
+             'NATIONAL CHAR(1)'),
+            (mysql.MSChar, [1], {'binary': True},
              'CHAR(1) BINARY'),
-            (mysql.MSChar, [1], {'ascii':True},
+            (mysql.MSChar, [1], {'ascii': True},
              'CHAR(1) ASCII'),
-            (mysql.MSChar, [1], {'unicode':True},
+            (mysql.MSChar, [1], {'unicode': True},
              'CHAR(1) UNICODE'),
-            (mysql.MSChar, [1], {'ascii':True, 'binary':True},
+            (mysql.MSChar, [1], {'ascii': True, 'binary': True},
              'CHAR(1) ASCII BINARY'),
-            (mysql.MSChar, [1], {'unicode':True, 'binary':True},
+            (mysql.MSChar, [1], {'unicode': True, 'binary': True},
              'CHAR(1) UNICODE BINARY'),
-            (mysql.MSChar, [1], {'charset':'utf8'},
+            (mysql.MSChar, [1], {'charset': 'utf8'},
              'CHAR(1) CHARACTER SET utf8'),
-            (mysql.MSChar, [1], {'charset':'utf8', 'binary':True},
+            (mysql.MSChar, [1], {'charset': 'utf8', 'binary': True},
              'CHAR(1) CHARACTER SET utf8 BINARY'),
-            (mysql.MSChar, [1], {'charset':'utf8', 'unicode':True},
+            (mysql.MSChar, [1], {'charset': 'utf8', 'unicode': True},
              'CHAR(1) CHARACTER SET utf8'),
-            (mysql.MSChar, [1], {'charset':'utf8', 'ascii':True},
+            (mysql.MSChar, [1], {'charset': 'utf8', 'ascii': True},
              'CHAR(1) CHARACTER SET utf8'),
             (mysql.MSChar, [1], {'collation': 'utf8_bin'},
              'CHAR(1) COLLATE utf8_bin'),
@@ -210,42 +214,42 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             (mysql.MSChar, [1], {'charset': 'utf8', 'binary': True},
              'CHAR(1) CHARACTER SET utf8 BINARY'),
             (mysql.MSChar, [1], {'charset': 'utf8', 'collation': 'utf8_bin',
-                              'binary': True},
+                                 'binary': True},
              'CHAR(1) CHARACTER SET utf8 COLLATE utf8_bin'),
-            (mysql.MSChar, [1], {'national':True},
+            (mysql.MSChar, [1], {'national': True},
              'NATIONAL CHAR(1)'),
-            (mysql.MSChar, [1], {'national':True, 'charset':'utf8'},
+            (mysql.MSChar, [1], {'national': True, 'charset': 'utf8'},
              'NATIONAL CHAR(1)'),
-            (mysql.MSChar, [1], {'national':True, 'charset':'utf8',
-                                'binary':True},
+            (mysql.MSChar, [1], {'national': True, 'charset': 'utf8',
+                                 'binary': True},
              'NATIONAL CHAR(1) BINARY'),
-            (mysql.MSChar, [1], {'national':True, 'binary':True,
-                                'unicode':True},
+            (mysql.MSChar, [1], {'national': True, 'binary': True,
+                                 'unicode': True},
              'NATIONAL CHAR(1) BINARY'),
-            (mysql.MSChar, [1], {'national':True, 'collation':'utf8_bin'},
+            (mysql.MSChar, [1], {'national': True, 'collation': 'utf8_bin'},
              'NATIONAL CHAR(1) COLLATE utf8_bin'),
 
-            (mysql.MSString, [1], {'charset':'utf8', 'collation':'utf8_bin'},
+            (mysql.MSString, [1], {'charset': 'utf8', 'collation': 'utf8_bin'},
              'VARCHAR(1) CHARACTER SET utf8 COLLATE utf8_bin'),
-            (mysql.MSString, [1], {'national':True, 'collation':'utf8_bin'},
+            (mysql.MSString, [1], {'national': True, 'collation': 'utf8_bin'},
              'NATIONAL VARCHAR(1) COLLATE utf8_bin'),
 
-            (mysql.MSTinyText, [], {'charset':'utf8', 'collation':'utf8_bin'},
+            (mysql.MSTinyText,
+             [],
+             {'charset': 'utf8', 'collation': 'utf8_bin'},
              'TINYTEXT CHARACTER SET utf8 COLLATE utf8_bin'),
 
-            (mysql.MSMediumText, [], {'charset':'utf8', 'binary':True},
+            (mysql.MSMediumText, [], {'charset': 'utf8', 'binary': True},
              'MEDIUMTEXT CHARACTER SET utf8 BINARY'),
 
-            (mysql.MSLongText, [], {'ascii':True},
+            (mysql.MSLongText, [], {'ascii': True},
              'LONGTEXT ASCII'),
 
-            (mysql.ENUM, ["foo", "bar"], {'unicode':True},
+            (mysql.ENUM, ["foo", "bar"], {'unicode': True},
              '''ENUM('foo','bar') UNICODE'''),
 
             (String, [20], {"collation": "utf8"}, 'VARCHAR(20) COLLATE utf8')
-
-
-           ]
+        ]
 
         for type_, args, kw, res in columns:
             type_inst = type_(*args, **kw)
@@ -256,8 +260,8 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             # test that repr() copies out all arguments
             self.assert_compile(
                 eval("mysql.%r" % type_inst)
-                    if type_ is not String
-                    else eval("%r" % type_inst),
+                if type_ is not String
+                else eval("%r" % type_inst),
                 res
             )
 
@@ -267,11 +271,10 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     @testing.provide_metadata
     def test_charset_collate_table(self):
         t = Table('foo', self.metadata,
-            Column('id', Integer),
-            Column('data', UnicodeText),
-            mysql_default_charset='utf8',
-            mysql_collate='utf8_bin'
-        )
+                  Column('id', Integer),
+                  Column('data', UnicodeText),
+                  mysql_default_charset='utf8',
+                  mysql_collate='utf8_bin')
         t.create()
         m2 = MetaData(testing.db)
         t2 = Table('foo', m2, autoload=True)
@@ -283,7 +286,8 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         # MySQLdb 1.2.3 and also need to pass either use_unicode=1
         # or charset=utf8 to the URL.
         t.insert().execute(id=1, data=u('some text'))
-        assert isinstance(testing.db.scalar(select([t.c.data])), util.text_type)
+        assert isinstance(testing.db.scalar(select([t.c.data])),
+                          util.text_type)
 
     def test_bit_50(self):
         """Exercise BIT types on 5.0+ (not valid for all engine types)"""
@@ -320,7 +324,7 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                 row = table.select().execute().first()
                 try:
                     self.assert_(list(row) == expected)
-                except:
+                except Exception:
                     print("Storing %s" % store)
                     print("Expected %s" % expected)
                     print("Found %s" % list(row))
@@ -359,8 +363,7 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             Column('b2', Boolean),
             Column('b3', mysql.MSTinyInteger(1)),
             Column('b4', mysql.MSTinyInteger(1, unsigned=True)),
-            Column('b5', mysql.MSTinyInteger),
-            )
+            Column('b5', mysql.MSTinyInteger))
         self.metadata.create_all()
         table = bool_table
 
@@ -393,8 +396,7 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             Column('b2', Boolean),
             Column('b3', BOOLEAN),
             Column('b4', BOOLEAN),
-            autoload=True,
-            )
+            autoload=True)
         eq_(colspec(table.c.b3), 'b3 BOOL')
         eq_(colspec(table.c.b4), 'b4 BOOL')
         roundtrip([None, None, None, None, None])
@@ -409,8 +411,8 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
     def test_timestamp_fsp(self):
         self.assert_compile(
-                mysql.TIMESTAMP(fsp=5),
-                "TIMESTAMP(5)"
+            mysql.TIMESTAMP(fsp=5),
+            "TIMESTAMP(5)"
         )
 
     def test_timestamp_defaults(self):
@@ -531,40 +533,37 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                 ]
             )
 
-
     def test_datetime_generic(self):
         self.assert_compile(
-                mysql.DATETIME(),
-                "DATETIME"
+            mysql.DATETIME(),
+            "DATETIME"
         )
 
     def test_datetime_fsp(self):
         self.assert_compile(
-                mysql.DATETIME(fsp=4),
-                "DATETIME(4)"
+            mysql.DATETIME(fsp=4),
+            "DATETIME(4)"
         )
 
-
     def test_time_generic(self):
         """"Exercise TIME."""
 
         self.assert_compile(
-                mysql.TIME(),
-                "TIME"
+            mysql.TIME(),
+            "TIME"
         )
 
     def test_time_fsp(self):
         self.assert_compile(
-                mysql.TIME(fsp=5),
-                "TIME(5)"
+            mysql.TIME(fsp=5),
+            "TIME(5)"
         )
 
     def test_time_result_processor(self):
         eq_(
             mysql.TIME().result_processor(None, None)(
-                    datetime.timedelta(seconds=35, minutes=517,
-                            microseconds=450
-                    )),
+                datetime.timedelta(seconds=35, minutes=517,
+                                   microseconds=450)),
             datetime.time(8, 37, 35, 450)
         )
 
@@ -572,8 +571,7 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     @testing.provide_metadata
     def test_time_roundtrip(self):
         t = Table('mysql_time', self.metadata,
-                Column('t1', mysql.TIME())
-            )
+                  Column('t1', mysql.TIME()))
         t.create()
         t.insert().values(t1=datetime.time(8, 37, 35)).execute()
         eq_(select([t.c.t1]).scalar(), datetime.time(8, 37, 35))
@@ -603,6 +601,49 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             eq_(colspec(table.c.y5), 'y5 YEAR(4)')
 
 
+class JSONTest(fixtures.TestBase):
+    __requires__ = ('json_type', )
+    __only_on__ = 'mysql'
+    __backend__ = True
+
+    @testing.provide_metadata
+    def test_reflection(self):
+
+        Table(
+            'mysql_json', self.metadata,
+            Column('foo', mysql.JSON)
+        )
+        self.metadata.create_all()
+
+        reflected = Table('mysql_json', MetaData(), autoload_with=testing.db)
+        is_(reflected.c.foo.type._type_affinity, sqltypes.JSON)
+        assert isinstance(reflected.c.foo.type, mysql.JSON)
+
+    @testing.provide_metadata
+    def test_rudimental_round_trip(self):
+        # note that test_suite has many more JSON round trip tests
+        # using the backend-agnostic JSON type
+
+        mysql_json = Table(
+            'mysql_json', self.metadata,
+            Column('foo', mysql.JSON)
+        )
+        self.metadata.create_all()
+
+        value = {
+            'json': {'foo': 'bar'},
+            'recs': ['one', 'two']
+        }
+
+        with testing.db.connect() as conn:
+            conn.execute(mysql_json.insert(), foo=value)
+
+            eq_(
+                conn.scalar(select([mysql_json.c.foo])),
+                value
+            )
+
+
 class EnumSetTest(
         fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
@@ -623,7 +664,8 @@ class EnumSetTest(
             'mysql_enum', self.metadata,
             Column('e1', e1),
             Column('e2', e2, nullable=False),
-            Column('e2generic', Enum("a", "b"), nullable=False),
+            Column('e2generic',
+                   Enum("a", "b", validate_strings=True), nullable=False),
             Column('e3', e3),
             Column('e4', e4,
                    nullable=False),
@@ -933,12 +975,12 @@ class EnumSetTest(
 
         eq_(
             t2.c.value.type.enums[0:2],
-            (u('réveillé'), u('drôle'))  # u'S’il') # eh ?
+            [u('réveillé'), u('drôle')]  # u'S’il') # eh ?
         )
 
         eq_(
             t2.c.value2.type.enums[0:2],
-            (u('réveillé'), u('drôle'))  # u'S’il') # eh ?
+            [u('réveillé'), u('drôle')]  # u'S’il') # eh ?
         )
 
     def test_enum_compile(self):
@@ -976,13 +1018,13 @@ class EnumSetTest(
         reflected = Table('mysql_enum', MetaData(testing.db),
                           autoload=True)
         for t in enum_table, reflected:
-            eq_(t.c.e1.type.enums, ("a",))
-            eq_(t.c.e2.type.enums, ("",))
-            eq_(t.c.e3.type.enums, ("a",))
-            eq_(t.c.e4.type.enums, ("",))
-            eq_(t.c.e5.type.enums, ("a", ""))
-            eq_(t.c.e6.type.enums, ("", "a"))
-            eq_(t.c.e7.type.enums, ("", "'a'", "b'b", "'"))
+            eq_(t.c.e1.type.enums, ["a"])
+            eq_(t.c.e2.type.enums, [""])
+            eq_(t.c.e3.type.enums, ["a"])
+            eq_(t.c.e4.type.enums, [""])
+            eq_(t.c.e5.type.enums, ["a", ""])
+            eq_(t.c.e6.type.enums, ["", "a"])
+            eq_(t.c.e7.type.enums, ["", "'a'", "b'b", "'"])
 
     @testing.provide_metadata
     @testing.exclude('mysql', '<', (5,))
@@ -1017,8 +1059,32 @@ class EnumSetTest(
             eq_(t.c.e6.type.values, ("", "a"))
             eq_(t.c.e7.type.values, ("", "'a'", "b'b", "'"))
 
+    @testing.provide_metadata
+    def test_broken_enum_returns_blanks(self):
+        t = Table(
+            'enum_missing',
+            self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('e1', sqltypes.Enum('one', 'two', 'three')),
+            Column('e2', mysql.ENUM('one', 'two', 'three'))
+        )
+        t.create()
+
+        with testing.db.connect() as conn:
+            conn.execute(t.insert(),
+                         {"e1": "nonexistent", "e2": "nonexistent"})
+            conn.execute(t.insert(), {"e1": "", "e2": ""})
+            conn.execute(t.insert(), {"e1": "two", "e2": "two"})
+            conn.execute(t.insert(), {"e1": None, "e2": None})
+
+            eq_(
+                conn.execute(
+                    select([t.c.e1, t.c.e2]).order_by(t.c.id)
+                ).fetchall(),
+                [("", ""), ("", ""), ("two", "two"), (None, None)]
+            )
+
 
 def colspec(c):
     return testing.db.dialect.ddl_compiler(
         testing.db.dialect, None).get_column_specification(c)
-
