@@ -232,6 +232,81 @@ class PoolTest(PoolTestBase):
         assert not c2.info
         assert 'foo2' in c.info
 
+    def test_rec_info(self):
+        p = self._queuepool_fixture(pool_size=1, max_overflow=0)
+
+        c = p.connect()
+        self.assert_(not c.record_info)
+        self.assert_(c.record_info is c._connection_record.record_info)
+
+        c.record_info['foo'] = 'bar'
+        c.close()
+        del c
+
+        c = p.connect()
+        self.assert_('foo' in c.record_info)
+
+        c.invalidate()
+        c = p.connect()
+        self.assert_('foo' in c.record_info)
+
+        c.record_info['foo2'] = 'bar2'
+        c.detach()
+        is_(c.record_info, None)
+        is_(c._connection_record, None)
+
+        c2 = p.connect()
+
+        assert c2.record_info
+        assert 'foo2' in c2.record_info
+
+    def test_rec_unconnected(self):
+        # test production of a _ConnectionRecord with an
+        # initially unconnected state.
+
+        dbapi = MockDBAPI()
+        p1 = pool.Pool(
+            creator=lambda: dbapi.connect('foo.db')
+        )
+
+        r1 = pool._ConnectionRecord(p1, connect=False)
+
+        assert not r1.connection
+        c1 = r1.get_connection()
+        is_(c1, r1.connection)
+
+    def test_rec_close_reopen(self):
+        # test that _ConnectionRecord.close() allows
+        # the record to be reusable
+        dbapi = MockDBAPI()
+        p1 = pool.Pool(
+            creator=lambda: dbapi.connect('foo.db')
+        )
+
+        r1 = pool._ConnectionRecord(p1)
+
+        c1 = r1.connection
+        c2 = r1.get_connection()
+        is_(c1, c2)
+
+        r1.close()
+
+        assert not r1.connection
+        eq_(
+            c1.mock_calls,
+            [call.close()]
+        )
+
+        c2 = r1.get_connection()
+
+        is_not_(c1, c2)
+        is_(c2, r1.connection)
+
+        eq_(
+            c2.mock_calls,
+            []
+        )
+
 
 class PoolDialectTest(PoolTestBase):
     def _dialect(self):
@@ -348,6 +423,66 @@ class PoolEventsTest(PoolTestBase):
 
         return p, canary
 
+    def _close_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = Mock()
+        event.listen(p, 'close', canary)
+
+        return p, canary
+
+    def _detach_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = Mock()
+        event.listen(p, 'detach', canary)
+
+        return p, canary
+
+    def _close_detached_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = Mock()
+        event.listen(p, 'close_detached', canary)
+
+        return p, canary
+
+    def test_close(self):
+        p, canary = self._close_event_fixture()
+
+        c1 = p.connect()
+
+        connection = c1.connection
+        rec = c1._connection_record
+
+        c1.close()
+
+        eq_(canary.mock_calls, [])
+
+        p.dispose()
+        eq_(canary.mock_calls, [call(connection, rec)])
+
+    def test_detach(self):
+        p, canary = self._detach_event_fixture()
+
+        c1 = p.connect()
+
+        connection = c1.connection
+        rec = c1._connection_record
+
+        c1.detach()
+
+        eq_(canary.mock_calls, [call(connection, rec)])
+
+    def test_detach_close(self):
+        p, canary = self._close_detached_event_fixture()
+
+        c1 = p.connect()
+
+        connection = c1.connection
+
+        c1.detach()
+
+        c1.close()
+        eq_(canary.mock_calls, [call(connection)])
+
     def test_first_connect_event(self):
         p, canary = self._first_connect_event_fixture()
 
