@@ -1,7 +1,7 @@
-from sqlalchemy import func, desc
-from sqlalchemy.orm import interfaces, create_session, joinedload, joinedload_all, \
-    subqueryload, subqueryload_all, aliased,\
-    class_mapper
+from sqlalchemy import func, desc, select
+from sqlalchemy.orm import (interfaces, create_session, joinedload,
+                            joinedload_all, subqueryload, subqueryload_all,
+                            aliased, class_mapper, with_polymorphic)
 from sqlalchemy import exc as sa_exc
 
 from sqlalchemy import testing
@@ -12,6 +12,7 @@ from ._poly_fixtures import Company, Person, Engineer, Manager, Boss, \
     _PolymorphicPolymorphic, _PolymorphicUnions, _PolymorphicJoins,\
     _PolymorphicAliasedJoins
 
+
 class _PolymorphicTestBase(object):
     __backend__ = True
 
@@ -22,7 +23,7 @@ class _PolymorphicTestBase(object):
         global companies, paperwork, machines
         people, engineers, managers, boss,\
             companies, paperwork, machines = \
-        cls.tables.people, cls.tables.engineers, \
+            cls.tables.people, cls.tables.engineers, \
             cls.tables.managers, cls.tables.boss,\
             cls.tables.companies, cls.tables.paperwork, cls.tables.machines
 
@@ -34,11 +35,10 @@ class _PolymorphicTestBase(object):
         global c1, c2, e1, e2, e3, b1, m1
         c1, c2, all_employees, c1_employees, c2_employees = \
             cls.c1, cls.c2, cls.all_employees, \
-                cls.c1_employees, cls.c2_employees
+            cls.c1_employees, cls.c2_employees
         e1, e2, e3, b1, m1 = \
             cls.e1, cls.e2, cls.e3, cls.b1, cls.m1
 
-
     def test_loads_at_once(self):
         """
         Test that all objects load from the full query, when
@@ -46,9 +46,12 @@ class _PolymorphicTestBase(object):
         """
 
         sess = create_session()
+
         def go():
-            eq_(sess.query(Person).all(), all_employees)
-        count = {'':14, 'Polymorphic':9}.get(self.select_type, 10)
+            eq_(
+                sess.query(Person).order_by(Person.person_id).all(),
+                all_employees)
+        count = {'': 14, 'Polymorphic': 9}.get(self.select_type, 10)
         self.assert_sql_count(testing.db, go, count)
 
     def test_primary_eager_aliasing_one(self):
@@ -56,20 +59,22 @@ class _PolymorphicTestBase(object):
         # not loading the subclass table, the joinedload doesn't happen.
 
         sess = create_session()
+
         def go():
-            eq_(sess.query(Person)
+            eq_(sess.query(Person).order_by(Person.person_id)
                     .options(joinedload(Engineer.machines))[1:3],
                 all_employees[1:3])
-        count = {'':6, 'Polymorphic':3}.get(self.select_type, 4)
+        count = {'': 6, 'Polymorphic': 3}.get(self.select_type, 4)
         self.assert_sql_count(testing.db, go, count)
 
     def test_primary_eager_aliasing_two(self):
         sess = create_session()
+
         def go():
-            eq_(sess.query(Person)
+            eq_(sess.query(Person).order_by(Person.person_id)
                     .options(subqueryload(Engineer.machines)).all(),
                 all_employees)
-        count = {'':14, 'Polymorphic':7}.get(self.select_type, 8)
+        count = {'': 14, 'Polymorphic': 7}.get(self.select_type, 8)
         self.assert_sql_count(testing.db, go, count)
 
     def test_primary_eager_aliasing_three(self):
@@ -77,17 +82,20 @@ class _PolymorphicTestBase(object):
         # assert the JOINs don't over JOIN
 
         sess = create_session()
+
         def go():
             eq_(sess.query(Person).with_polymorphic('*')
+                    .order_by(Person.person_id)
                     .options(joinedload(Engineer.machines))[1:3],
                 all_employees[1:3])
         self.assert_sql_count(testing.db, go, 3)
 
-        eq_(sess.query(Person).with_polymorphic('*')
+        eq_(
+            select([func.count('*')]).select_from(
+                sess.query(Person).with_polymorphic('*')
                 .options(joinedload(Engineer.machines))
-                .limit(2).offset(1).with_labels()
-                .subquery().count().scalar(),
-            2)
+                .limit(2).offset(1).with_labels().subquery()
+            ).scalar(), 2)
 
     def test_get_one(self):
         """
@@ -183,6 +191,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_nonaliased_two(self):
         sess = create_session()
         eq_(sess.query(Person)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=False)
                 .filter(Paperwork.description.like('%#2%')).all(),
             [e1, m1])
@@ -190,6 +199,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_nonaliased_three(self):
         sess = create_session()
         eq_(sess.query(Engineer)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=False)
                 .filter(Paperwork.description.like('%#2%')).all(),
             [e1])
@@ -197,6 +207,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_nonaliased_four(self):
         sess = create_session()
         eq_(sess.query(Person)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=False)
                 .filter(Person.name.like('%dog%'))
                 .filter(Paperwork.description.like('%#2%')).all(),
@@ -205,6 +216,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_aliased_one(self):
         sess = create_session()
         eq_(sess.query(Person)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=True)
                 .filter(Paperwork.description.like('%review%')).all(),
             [b1, m1])
@@ -212,6 +224,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_aliased_two(self):
         sess = create_session()
         eq_(sess.query(Person)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=True)
                 .filter(Paperwork.description.like('%#2%')).all(),
             [e1, m1])
@@ -219,6 +232,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_aliased_three(self):
         sess = create_session()
         eq_(sess.query(Engineer)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=True)
                 .filter(Paperwork.description.like('%#2%')).all(),
             [e1])
@@ -226,6 +240,7 @@ class _PolymorphicTestBase(object):
     def test_join_from_polymorphic_aliased_four(self):
         sess = create_session()
         eq_(sess.query(Person)
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=True)
                 .filter(Person.name.like('%dog%'))
                 .filter(Paperwork.description.like('%#2%')).all(),
@@ -235,6 +250,7 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         eq_(sess.query(Person)
                 .with_polymorphic(Manager)
+                .order_by(Person.person_id)
                 .join('paperwork')
                 .filter(Paperwork.description.like('%review%')).all(),
             [b1, m1])
@@ -243,6 +259,7 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         eq_(sess.query(Person)
                 .with_polymorphic([Manager, Engineer])
+                .order_by(Person.person_id)
                 .join('paperwork')
                 .filter(Paperwork.description.like('%#2%')).all(),
             [e1, m1])
@@ -251,12 +268,12 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         eq_(sess.query(Person)
                 .with_polymorphic([Manager, Engineer])
+                .order_by(Person.person_id)
                 .join('paperwork')
                 .filter(Person.name.like('%dog%'))
                 .filter(Paperwork.description.like('%#2%')).all(),
             [m1])
 
-
     def test_join_from_with_polymorphic_aliased_one(self):
         sess = create_session()
         eq_(sess.query(Person)
@@ -269,6 +286,7 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         eq_(sess.query(Person)
                 .with_polymorphic([Manager, Engineer])
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=True)
                 .filter(Paperwork.description.like('%#2%')).all(),
             [e1, m1])
@@ -277,6 +295,7 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         eq_(sess.query(Person)
                 .with_polymorphic([Manager, Engineer])
+                .order_by(Person.person_id)
                 .join('paperwork', aliased=True)
                 .filter(Person.name.like('%dog%'))
                 .filter(Paperwork.description.like('%#2%')).all(),
@@ -326,14 +345,17 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         any_ = Engineer.machines.any(
             Machine.name == "Commodore 64")
-        eq_(sess.query(Person).filter(any_).all(), [e2, e3])
+        eq_(
+            sess.query(Person).order_by(Person.person_id).filter(any_).all(),
+            [e2, e3])
 
     def test_polymorphic_any_nine(self):
         sess = create_session()
         any_ = Person.paperwork.any(
             Paperwork.description == "review #2")
-        eq_(sess.query(Person).filter(any_).all(), [m1])
-
+        eq_(
+            sess.query(Person).order_by(Person.person_id).filter(any_).all(),
+            [m1])
 
     def test_join_from_columns_or_subclass_one(self):
         sess = create_session()
@@ -498,13 +520,12 @@ class _PolymorphicTestBase(object):
                 .all(),
             expected)
 
-
     def test_subclass_option_pathing(self):
         from sqlalchemy.orm import defer
         sess = create_session()
         dilbert = sess.query(Person).\
-                options(defer(Engineer.machines, Machine.name)).\
-                filter(Person.name == 'dilbert').first()
+            options(defer(Engineer.machines, Machine.name)).\
+            filter(Person.name == 'dilbert').first()
         m = dilbert.machines[0]
         assert 'name' not in m.__dict__
         eq_(m.name, 'IBM ThinkPad')
@@ -529,6 +550,7 @@ class _PolymorphicTestBase(object):
 
     def test_with_polymorphic_one(self):
         sess = create_session()
+
         def go():
             eq_(sess.query(Person)
                     .with_polymorphic(Engineer)
@@ -536,26 +558,28 @@ class _PolymorphicTestBase(object):
                 self._emps_wo_relationships_fixture()[0:1])
         self.assert_sql_count(testing.db, go, 1)
 
-
     def test_with_polymorphic_two(self):
         sess = create_session()
+
         def go():
             eq_(sess.query(Person)
-                    .with_polymorphic('*').all(),
+                    .with_polymorphic('*').order_by(Person.person_id).all(),
                 self._emps_wo_relationships_fixture())
         self.assert_sql_count(testing.db, go, 1)
 
     def test_with_polymorphic_three(self):
         sess = create_session()
+
         def go():
             eq_(sess.query(Person)
                     .with_polymorphic(Engineer).
-                    order_by(Person.person_id).all(),
+                order_by(Person.person_id).all(),
                 self._emps_wo_relationships_fixture())
         self.assert_sql_count(testing.db, go, 3)
 
     def test_with_polymorphic_four(self):
         sess = create_session()
+
         def go():
             eq_(sess.query(Person)
                     .with_polymorphic(
@@ -568,6 +592,7 @@ class _PolymorphicTestBase(object):
 
     def test_with_polymorphic_five(self):
         sess = create_session()
+
         def go():
             # limit the polymorphic join down to just "Person",
             # overriding select_table
@@ -580,32 +605,34 @@ class _PolymorphicTestBase(object):
         sess = create_session()
 
         assert_raises(sa_exc.InvalidRequestError,
-            sess.query(Person).with_polymorphic, Paperwork)
+                      sess.query(Person).with_polymorphic, Paperwork)
         assert_raises(sa_exc.InvalidRequestError,
-            sess.query(Engineer).with_polymorphic, Boss)
+                      sess.query(Engineer).with_polymorphic, Boss)
         assert_raises(sa_exc.InvalidRequestError,
-            sess.query(Engineer).with_polymorphic, Person)
+                      sess.query(Engineer).with_polymorphic, Person)
 
     def test_with_polymorphic_seven(self):
         sess = create_session()
         # compare to entities without related collections to prevent
         # additional lazy SQL from firing on loaded entities
-        eq_(sess.query(Person).with_polymorphic('*').all(),
+        eq_(sess.query(Person).with_polymorphic('*').
+            order_by(Person.person_id).all(),
             self._emps_wo_relationships_fixture())
 
-
     def test_relationship_to_polymorphic_one(self):
         expected = self._company_with_emps_machines_fixture()
         sess = create_session()
+
         def go():
             # test load Companies with lazy load to 'employees'
             eq_(sess.query(Company).all(), expected)
-        count = {'':10, 'Polymorphic':5}.get(self.select_type, 6)
+        count = {'': 10, 'Polymorphic': 5}.get(self.select_type, 6)
         self.assert_sql_count(testing.db, go, count)
 
     def test_relationship_to_polymorphic_two(self):
         expected = self._company_with_emps_machines_fixture()
         sess = create_session()
+
         def go():
             # with #2438, of_type() is recognized.  This
             # overrides the with_polymorphic of the mapper
@@ -618,7 +645,7 @@ class _PolymorphicTestBase(object):
                 expected)
 
         # in the old case, we would get this
-        #count = {'':7, 'Polymorphic':1}.get(self.select_type, 2)
+        # count = {'':7, 'Polymorphic':1}.get(self.select_type, 2)
 
         # query one is company->Person/Engineer->Machines
         # query two is managers + boss for row #3
@@ -631,6 +658,7 @@ class _PolymorphicTestBase(object):
         sess = create_session()
 
         sess = create_session()
+
         def go():
             eq_(sess.query(Company)
                     .options(subqueryload_all(
@@ -641,7 +669,7 @@ class _PolymorphicTestBase(object):
 
         # the old case where subqueryload_all
         # didn't work with of_tyoe
-        #count = { '':8, 'Joins':4, 'Unions':4, 'Polymorphic':3,
+        # count = { '':8, 'Joins':4, 'Unions':4, 'Polymorphic':3,
         #    'AliasedJoins':4}[self.select_type]
 
         # query one is company->Person/Engineer->Machines
@@ -654,7 +682,6 @@ class _PolymorphicTestBase(object):
         count = 5
         self.assert_sql_count(testing.db, go, count)
 
-
     def test_joinedload_on_subclass(self):
         sess = create_session()
         expected = [
@@ -673,7 +700,7 @@ class _PolymorphicTestBase(object):
                     .with_polymorphic('*')
                     .options(joinedload(Engineer.machines))
                     .filter(Person.name == 'dilbert').all(),
-              expected)
+                expected)
         self.assert_sql_count(testing.db, go, 1)
 
     def test_subqueryload_on_subclass(self):
@@ -687,13 +714,14 @@ class _PolymorphicTestBase(object):
                 machines=[
                     Machine(name="IBM ThinkPad"),
                     Machine(name="IPhone")])]
+
         def go():
             # test load People with subqueryload to engineers + machines
             eq_(sess.query(Person)
                     .with_polymorphic('*')
                     .options(subqueryload(Engineer.machines))
                     .filter(Person.name == 'dilbert').all(),
-              expected)
+                expected)
         self.assert_sql_count(testing.db, go, 2)
 
     def test_query_subclass_join_to_base_relationship(self):
@@ -756,7 +784,6 @@ class _PolymorphicTestBase(object):
                 .filter(Machine.name.ilike("%thinkpad%")).all(),
             [c1])
 
-
     def test_join_to_subclass_eight(self):
         sess = create_session()
         eq_(sess.query(Person)
@@ -920,7 +947,6 @@ class _PolymorphicTestBase(object):
                 .filter(Paperwork.description.like('%#%')).all(),
             [c1, c2])
 
-
     def test_explicit_polymorphic_join_one(self):
         sess = create_session()
 
@@ -946,9 +972,11 @@ class _PolymorphicTestBase(object):
 
     def test_filter_on_baseclass(self):
         sess = create_session()
-        eq_(sess.query(Person).all(), all_employees)
-        eq_(sess.query(Person).first(), all_employees[0])
-        eq_(sess.query(Person)
+        eq_(sess.query(Person).order_by(Person.person_id).all(), all_employees)
+        eq_(
+            sess.query(Person).order_by(Person.person_id).first(),
+            all_employees[0])
+        eq_(sess.query(Person).order_by(Person.person_id)
                 .filter(Person.person_id == e2.person_id).one(),
             e2)
 
@@ -956,6 +984,7 @@ class _PolymorphicTestBase(object):
         sess = create_session()
         palias = aliased(Person)
         eq_(sess.query(palias)
+                .order_by(palias.person_id)
                 .filter(palias.name.in_(['dilbert', 'wally'])).all(),
             [e1, e2])
 
@@ -1010,7 +1039,7 @@ class _PolymorphicTestBase(object):
                 primary_language='java',
                 person_id=1,
                 type='engineer'),
-            'MegaCorp, Inc.'),
+             'MegaCorp, Inc.'),
             (Engineer(
                 status='regular engineer',
                 engineer_name='wally',
@@ -1019,7 +1048,7 @@ class _PolymorphicTestBase(object):
                 primary_language='c++',
                 person_id=2,
                 type='engineer'),
-            'MegaCorp, Inc.'),
+             'MegaCorp, Inc.'),
             (Engineer(
                 status='elbonian engineer',
                 engineer_name='vlad',
@@ -1028,9 +1057,10 @@ class _PolymorphicTestBase(object):
                 primary_language='cobol',
                 person_id=5,
                 type='engineer'),
-            'Elbonia, Inc.')]
+             'Elbonia, Inc.')]
         eq_(sess.query(Engineer, Company.name)
                 .join(Company.employees)
+                .order_by(Person.person_id)
                 .filter(Person.type == 'engineer').all(),
             expected)
 
@@ -1158,7 +1188,7 @@ class _PolymorphicTestBase(object):
                 .order_by(Company.name).all(),
             expected)
 
-    #def test_mixed_entities(self):
+    # def test_mixed_entities(self):
     #    sess = create_session()
         # TODO: I think raise error on these for now.  different
         # inheritance/loading schemes have different results here,
@@ -1168,7 +1198,7 @@ class _PolymorphicTestBase(object):
         #    sess.query(Person.name, Engineer.primary_language).all(),
         #    [])
 
-    #def test_mixed_entities(self):
+    # def test_mixed_entities(self):
     #    sess = create_session()
         # eq_(sess.query(
         #             Person.name,
@@ -1252,6 +1282,45 @@ class _PolymorphicTestBase(object):
         assert row.name == 'dilbert'
         assert row.primary_language == 'java'
 
+    def test_correlation_one(self):
+        sess = create_session()
+
+        # unfortunately this pattern can't yet work for PolymorphicAliased
+        # and PolymorphicUnions, because the subquery does not compile
+        # out including the polymorphic selectable; only if Person is in
+        # the query() list does that happen.
+        eq_(sess.query(Person.name)
+                .filter(
+                    sess.query(Company.name).
+                    filter(Company.company_id == Person.company_id).
+                    correlate(Person).as_scalar() == "Elbonia, Inc.").all(),
+            [(e3.name, )])
+
+    def test_correlation_two(self):
+        sess = create_session()
+
+        paliased = aliased(Person)
+
+        eq_(sess.query(paliased.name)
+                .filter(
+                    sess.query(Company.name).
+                    filter(Company.company_id == paliased.company_id).
+                    correlate(paliased).as_scalar() == "Elbonia, Inc.").all(),
+            [(e3.name, )])
+
+    def test_correlation_three(self):
+        sess = create_session()
+
+        paliased = aliased(Person, flat=True)
+
+        eq_(sess.query(paliased.name)
+                .filter(
+                    sess.query(Company.name).
+                    filter(Company.company_id == paliased.company_id).
+                    correlate(paliased).as_scalar() == "Elbonia, Inc.").all(),
+            [(e3.name, )])
+
+
 class PolymorphicTest(_PolymorphicTestBase, _Polymorphic):
     def test_join_to_subclass_four(self):
         sess = create_session()
@@ -1268,6 +1337,31 @@ class PolymorphicTest(_PolymorphicTestBase, _Polymorphic):
                 .filter(Machine.name.ilike("%ibm%")).all(),
             [e1, e3])
 
+    def test_correlation_w_polymorphic(self):
+
+        sess = create_session()
+
+        p_poly = with_polymorphic(Person, '*')
+
+        eq_(sess.query(p_poly.name)
+                .filter(
+                    sess.query(Company.name).
+                    filter(Company.company_id == p_poly.company_id).
+                    correlate(p_poly).as_scalar() == "Elbonia, Inc.").all(),
+            [(e3.name, )])
+
+    def test_correlation_w_polymorphic_flat(self):
+
+        sess = create_session()
+
+        p_poly = with_polymorphic(Person, '*', flat=True)
+
+        eq_(sess.query(p_poly.name)
+                .filter(
+                    sess.query(Company.name).
+                    filter(Company.company_id == p_poly.company_id).
+                    correlate(p_poly).as_scalar() == "Elbonia, Inc.").all(),
+            [(e3.name, )])
 
     def test_join_to_subclass_ten(self):
         pass
@@ -1285,7 +1379,8 @@ class PolymorphicTest(_PolymorphicTestBase, _Polymorphic):
         pass
 
 
-class PolymorphicPolymorphicTest(_PolymorphicTestBase, _PolymorphicPolymorphic):
+class PolymorphicPolymorphicTest(
+        _PolymorphicTestBase, _PolymorphicPolymorphic):
     __dialect__ = 'default'
 
     def test_aliased_not_polluted_by_join(self):
@@ -1295,18 +1390,17 @@ class PolymorphicPolymorphicTest(_PolymorphicTestBase, _PolymorphicPolymorphic):
         sess = create_session()
         palias = aliased(Person)
         self.assert_compile(
-            sess.query(palias, Company.name)
-                .join(Person, Company.employees)
-                .filter(palias.name == 'dilbert'),
+            sess.query(palias, Company.name).order_by(palias.person_id).
+            join(Person, Company.employees).filter(palias.name == 'dilbert'),
             "SELECT anon_1.people_person_id AS anon_1_people_person_id, "
             "anon_1.people_company_id AS anon_1_people_company_id, "
             "anon_1.people_name AS anon_1_people_name, "
             "anon_1.people_type AS anon_1_people_type, "
             "anon_1.engineers_person_id AS anon_1_engineers_person_id, "
             "anon_1.engineers_status AS anon_1_engineers_status, "
-            "anon_1.engineers_engineer_name AS anon_1_engineers_engineer_name, "
+            "anon_1.engineers_engineer_name AS anon_1_engineers_engineer_name, "  # noqa
             "anon_1.engineers_primary_language AS "
-                "anon_1_engineers_primary_language, "
+            "anon_1_engineers_primary_language, "
             "anon_1.managers_person_id AS anon_1_managers_person_id, "
             "anon_1.managers_status AS anon_1_managers_status, "
             "anon_1.managers_manager_name AS anon_1_managers_manager_name, "
@@ -1314,41 +1408,39 @@ class PolymorphicPolymorphicTest(_PolymorphicTestBase, _PolymorphicPolymorphic):
             "anon_1.boss_golf_swing AS anon_1_boss_golf_swing, "
             "companies.name AS companies_name "
             "FROM (SELECT people.person_id AS people_person_id, "
-                "people.company_id AS people_company_id, "
-                "people.name AS people_name, people.type AS people_type, "
-                "engineers.person_id AS engineers_person_id, "
-                "engineers.status AS engineers_status, "
-                "engineers.engineer_name AS engineers_engineer_name, "
-                "engineers.primary_language AS engineers_primary_language, "
-                "managers.person_id AS managers_person_id, "
-                "managers.status AS managers_status, "
-                "managers.manager_name AS managers_manager_name, "
-                "boss.boss_id AS boss_boss_id, "
-                "boss.golf_swing AS boss_golf_swing "
-                "FROM people LEFT OUTER JOIN engineers "
-                "ON people.person_id = engineers.person_id "
-                "LEFT OUTER JOIN managers "
-                "ON people.person_id = managers.person_id LEFT OUTER JOIN boss "
-                "ON managers.person_id = boss.boss_id) AS anon_1, "
-                "companies JOIN "
-                "(people LEFT OUTER JOIN engineers "
-                    "ON people.person_id = engineers.person_id "
-                    "LEFT OUTER JOIN managers "
-                    "ON people.person_id = managers.person_id "
-                    "LEFT OUTER JOIN boss ON managers.person_id = boss.boss_id) "
-                "ON companies.company_id = people.company_id "
-                "WHERE anon_1.people_name = :people_name_1 "
-                "ORDER BY anon_1.people_person_id"
-            )
+            "people.company_id AS people_company_id, "
+            "people.name AS people_name, people.type AS people_type, "
+            "engineers.person_id AS engineers_person_id, "
+            "engineers.status AS engineers_status, "
+            "engineers.engineer_name AS engineers_engineer_name, "
+            "engineers.primary_language AS engineers_primary_language, "
+            "managers.person_id AS managers_person_id, "
+            "managers.status AS managers_status, "
+            "managers.manager_name AS managers_manager_name, "
+            "boss.boss_id AS boss_boss_id, "
+            "boss.golf_swing AS boss_golf_swing "
+            "FROM people LEFT OUTER JOIN engineers "
+            "ON people.person_id = engineers.person_id "
+            "LEFT OUTER JOIN managers "
+            "ON people.person_id = managers.person_id LEFT OUTER JOIN boss "
+            "ON managers.person_id = boss.boss_id) AS anon_1, "
+            "companies JOIN "
+            "(people LEFT OUTER JOIN engineers "
+            "ON people.person_id = engineers.person_id "
+            "LEFT OUTER JOIN managers "
+            "ON people.person_id = managers.person_id "
+            "LEFT OUTER JOIN boss ON managers.person_id = boss.boss_id) "
+            "ON companies.company_id = people.company_id "
+            "WHERE anon_1.people_name = :people_name_1 "
+            "ORDER BY anon_1.people_person_id")
 
     def test_flat_aliased_w_select_from(self):
         sess = create_session()
         palias = aliased(Person, flat=True)
         self.assert_compile(
-            sess.query(palias, Company.name)
-                .select_from(palias)
-                .join(Person, Company.employees)
-                .filter(palias.name == 'dilbert'),
+            sess.query(palias, Company.name).
+            select_from(palias).order_by(palias.person_id).join(
+                Person, Company.employees).filter(palias.name == 'dilbert'),
             "SELECT people_1.person_id AS people_1_person_id, "
             "people_1.company_id AS people_1_company_id, "
             "people_1.name AS people_1_name, people_1.type AS people_1_type, "
@@ -1371,18 +1463,26 @@ class PolymorphicPolymorphicTest(_PolymorphicTestBase, _PolymorphicPolymorphic):
             "ON managers_1.person_id = boss_1.boss_id, "
             "companies JOIN (people LEFT OUTER JOIN engineers "
             "ON people.person_id = engineers.person_id "
-            "LEFT OUTER JOIN managers ON people.person_id = managers.person_id "
+            "LEFT OUTER JOIN managers "
+            "ON people.person_id = managers.person_id "
             "LEFT OUTER JOIN boss ON managers.person_id = boss.boss_id) "
             "ON companies.company_id = people.company_id "
-            "WHERE people_1.name = :name_1 ORDER BY people_1.person_id"
-        )
+            "WHERE people_1.name = :name_1 ORDER BY people_1.person_id")
 
 
 class PolymorphicUnionsTest(_PolymorphicTestBase, _PolymorphicUnions):
-    pass
 
-class PolymorphicAliasedJoinsTest(_PolymorphicTestBase, _PolymorphicAliasedJoins):
-    pass
+    @testing.fails()
+    def test_correlation_one(self):
+        super(PolymorphicUnionsTest, self).test_correlation_one()
+
+
+class PolymorphicAliasedJoinsTest(
+        _PolymorphicTestBase, _PolymorphicAliasedJoins):
+    @testing.fails()
+    def test_correlation_one(self):
+        super(PolymorphicAliasedJoinsTest, self).test_correlation_one()
+
 
 class PolymorphicJoinsTest(_PolymorphicTestBase, _PolymorphicJoins):
     pass
