@@ -21,12 +21,15 @@ from sqlalchemy.testing.exclusions import \
      LambdaPredicate,\
      requires_tag
 
+
 def no_support(db, reason):
     return SpecPredicate(db, description=reason)
 
+
 def exclude(db, op, spec, description=None):
     return SpecPredicate(db, op, spec, description=description)
 
+
 class DefaultRequirements(SuiteRequirements):
     @property
     def deferrable_or_no_constraints(self):
@@ -44,6 +47,14 @@ class DefaultRequirements(SuiteRequirements):
 
         return exclusions.open()
 
+    @property
+    def enforces_check_constraints(self):
+        """Target database must also enforce check constraints."""
+
+        return self.check_constraints + fails_on(
+            ['mysql'], "check constraints don't enforce"
+        )
+
     @property
     def named_constraints(self):
         """target database must support names for constraints."""
@@ -84,6 +95,9 @@ class DefaultRequirements(SuiteRequirements):
 
         return only_on(['oracle'])
 
+    @property
+    def foreign_key_constraint_option_reflection(self):
+        return only_on(['postgresql', 'mysql', 'sqlite'])
 
     @property
     def unbounded_varchar(self):
@@ -110,21 +124,20 @@ class DefaultRequirements(SuiteRequirements):
         without being in the context of a typed column.
 
         """
-        return skip_if(["firebird", "mssql+mxodbc"],
-                "not supported by driver")
+        return skip_if(["firebird", "mssql+mxodbc"], "not supported by driver")
 
     @property
     def identity(self):
         """Target database must support GENERATED AS IDENTITY or a facsimile.
 
         Includes GENERATED AS IDENTITY, AUTOINCREMENT, AUTO_INCREMENT, or other
-        column DDL feature that fills in a DB-generated identifier at INSERT-time
-        without requiring pre-execution of a SEQUENCE or other artifact.
+        column DDL feature that fills in a DB-generated identifier at
+        INSERT-time without requiring pre-execution of a SEQUENCE or other
+        artifact.
 
         """
         return skip_if(["firebird", "oracle", "postgresql", "sybase"],
-                "not supported by database"
-            )
+                       "not supported by database")
 
     @property
     def temporary_tables(self):
@@ -142,13 +155,12 @@ class DefaultRequirements(SuiteRequirements):
         """Target database must support tables that can automatically generate
         PKs assuming they were reflected.
 
-        this is essentially all the DBs in "identity" plus Postgresql, which
-        has SERIAL support.  FB and Oracle (and sybase?) require the Sequence to
-        be explicitly added, including if the table was reflected.
+        this is essentially all the DBs in "identity" plus PostgreSQL, which
+        has SERIAL support.  FB and Oracle (and sybase?) require the Sequence
+        to be explicitly added, including if the table was reflected.
         """
         return skip_if(["firebird", "oracle", "sybase"],
-                "not supported by database"
-            )
+                       "not supported by database")
 
     @property
     def insert_from_select(self):
@@ -167,9 +179,7 @@ class DefaultRequirements(SuiteRequirements):
         """target database/driver can allow BLOB/BINARY fields to be compared
         against a bound parameter value.
         """
-        return skip_if(["oracle", "mssql"],
-                "not supported by database/driver"
-            )
+        return skip_if(["oracle", "mssql"], "not supported by database/driver")
 
     @property
     def binary_literals(self):
@@ -187,9 +197,7 @@ class DefaultRequirements(SuiteRequirements):
         # adding mssql here since it doesn't support comparisons either,
         # have observed generally bad behavior with binary / mssql.
 
-        return skip_if(["oracle", "mssql"],
-                "not supported by database/driver"
-            )
+        return skip_if(["oracle", "mssql"], "not supported by database/driver")
 
     @property
     def independent_cursors(self):
@@ -200,35 +208,34 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def independent_connections(self):
-        """Target must support simultaneous, independent database connections."""
+        """
+        Target must support simultaneous, independent database connections.
+        """
 
-        # This is also true of some configurations of UnixODBC and probably win32
-        # ODBC as well.
+        # This is also true of some configurations of UnixODBC and probably
+        # win32 ODBC as well.
         return skip_if([
-                    no_support("sqlite",
-                            "independent connections disabled "
-                                "when :memory: connections are used"),
-                    exclude("mssql", "<", (9, 0, 0),
-                            "SQL Server 2005+ is required for "
-                                "independent connections"
-                        )
-                    ]
-                )
+            no_support("sqlite",
+                       "independent connections disabled "
+                       "when :memory: connections are used"),
+            exclude("mssql", "<", (9, 0, 0),
+                    "SQL Server 2005+ is required for "
+                    "independent connections")])
 
     @property
     def updateable_autoincrement_pks(self):
         """Target must support UPDATE on autoincrement/integer primary key."""
 
         return skip_if(["mssql", "sybase"],
-                "IDENTITY columns can't be updated")
+                       "IDENTITY columns can't be updated")
 
     @property
     def isolation_level(self):
         return only_on(
-                    ('postgresql', 'sqlite', 'mysql'),
-                    "DBAPI has no isolation level support"
-                ) + fails_on('postgresql+pypostgresql',
-                          'pypostgresql bombs on multiple isolation level calls')
+            ('postgresql', 'sqlite', 'mysql', 'mssql'),
+            "DBAPI has no isolation level support") \
+            + fails_on('postgresql+pypostgresql',
+                       'pypostgresql bombs on multiple isolation level calls')
 
     @property
     def row_triggers(self):
@@ -241,7 +248,7 @@ class DefaultRequirements(SuiteRequirements):
 
             # huh?  TODO: implement triggers for PG tests, remove this
             no_support('postgresql',
-                    'PG triggers need to be implemented for tests'),
+                       'PG triggers need to be implemented for tests'),
         ])
 
     @property
@@ -250,15 +257,14 @@ class DefaultRequirements(SuiteRequirements):
         correlates to the parent."""
 
         return skip_if("oracle", 'Raises "ORA-01799: a column may not be '
-                    'outer-joined to a subquery"')
+                       'outer-joined to a subquery"')
 
     @property
     def update_from(self):
         """Target must support UPDATE..FROM syntax"""
 
         return only_on(['postgresql', 'mssql', 'mysql'],
-                "Backend does not support UPDATE..FROM")
-
+                       "Backend does not support UPDATE..FROM")
 
     @property
     def update_where_target_in_subquery(self):
@@ -273,8 +279,9 @@ class DefaultRequirements(SuiteRequirements):
                 FROM documents GROUP BY documents.user_id
             )
         """
-        return fails_if('mysql', 'MySQL error 1093 "Cant specify target table '
-                                        'for update in FROM clause"')
+        return fails_if('mysql',
+                        'MySQL error 1093 "Cant specify target table '
+                        'for update in FROM clause"')
 
     @property
     def savepoints(self):
@@ -297,7 +304,6 @@ class DefaultRequirements(SuiteRequirements):
         named 'test_schema'."""
 
         return skip_if([
-                    "sqlite",
                     "firebird"
                 ], "no schema support")
 
@@ -317,6 +323,17 @@ class DefaultRequirements(SuiteRequirements):
                     "sqlite"
                 )
 
+    @property
+    def unique_constraint_reflection_no_index_overlap(self):
+        return self.unique_constraint_reflection + skip_if("mysql")
+
+    @property
+    def check_constraint_reflection(self):
+        return fails_on_everything_except(
+                    "postgresql",
+                    "sqlite"
+                )
+
     @property
     def temp_table_names(self):
         """target dialect supports listing of temporary table names"""
@@ -332,8 +349,7 @@ class DefaultRequirements(SuiteRequirements):
     def update_nowait(self):
         """Target database must support SELECT...FOR UPDATE NOWAIT"""
         return skip_if(["firebird", "mssql", "mysql", "sqlite", "sybase"],
-                "no FOR UPDATE NOWAIT support"
-            )
+                       "no FOR UPDATE NOWAIT support")
 
     @property
     def subqueries(self):
@@ -341,6 +357,14 @@ class DefaultRequirements(SuiteRequirements):
 
         return skip_if(exclude('mysql', '<', (4, 1, 1)), 'no subquery support')
 
+    @property
+    def ctes(self):
+        """Target database supports CTEs"""
+
+        return only_if(
+            ['postgresql', 'mssql']
+        )
+
     @property
     def mod_operator_as_percent_sign(self):
         """target database must use a plain percent '%' as the 'modulus'
@@ -365,6 +389,32 @@ class DefaultRequirements(SuiteRequirements):
                 "firebird", "mysql", "sybase",
             ], 'no support for EXCEPT')
 
+    @property
+    def parens_in_union_contained_select_w_limit_offset(self):
+        """Target database must support parenthesized SELECT in UNION
+        when LIMIT/OFFSET is specifically present.
+
+        E.g. (SELECT ...) UNION (SELECT ..)
+
+        This is known to fail on SQLite.
+
+        """
+        return fails_if('sqlite')
+
+    @property
+    def parens_in_union_contained_select_wo_limit_offset(self):
+        """Target database must support parenthesized SELECT in UNION
+        when OFFSET/LIMIT is specifically not present.
+
+        E.g. (SELECT ... LIMIT ..) UNION (SELECT .. OFFSET ..)
+
+        This is known to fail on SQLite.  It also fails on Oracle
+        because without LIMIT/OFFSET, there is currently no step that
+        creates an additional subquery.
+
+        """
+        return fails_if(['sqlite', 'oracle'])
+
     @property
     def offset(self):
         """Target database must support some method of adding OFFSET or
@@ -386,29 +436,28 @@ class DefaultRequirements(SuiteRequirements):
         return skip_if([
             no_support('firebird', 'no SA implementation'),
             no_support('mssql', 'two-phase xact not supported by drivers'),
-            no_support('oracle', 'two-phase xact not implemented in SQLA/oracle'),
+            no_support('oracle',
+                       'two-phase xact not implemented in SQLA/oracle'),
             no_support('drizzle', 'two-phase xact not supported by database'),
             no_support('sqlite', 'two-phase xact not supported by database'),
-            no_support('sybase', 'two-phase xact not supported by drivers/SQLA'),
+            no_support('sybase',
+                       'two-phase xact not supported by drivers/SQLA'),
             no_support('postgresql+zxjdbc',
-                    'FIXME: JDBC driver confuses the transaction state, may '
-                       'need separate XA implementation'),
-            exclude('mysql', '<', (5, 0, 3),
-                        'two-phase xact not supported by database'),
-            ])
+                       'FIXME: JDBC driver confuses the transaction state, '
+                       'may need separate XA implementation'),
+            no_support('mysql',
+                       'recent MySQL communiity editions have too many issues '
+                       '(late 2016), disabling for now')])
 
     @property
     def two_phase_recovery(self):
         return self.two_phase_transactions + (
-            exclusions.fails_if(
-               lambda config:  config.db.name == 'mysql' and (
-                        'MariaDB' in config.db.dialect.server_version_info or
-                        config.db.dialect.server_version_info < (5, 7)
-               )
+            skip_if(
+               "mysql",
+               "crashes on most mariadb and mysql versions"
             )
         )
 
-
     @property
     def views(self):
         """Target database must support VIEWs."""
@@ -417,7 +466,9 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def empty_strings_varchar(self):
-        """target database can persist/return an empty string with a varchar."""
+        """
+        target database can persist/return an empty string with a varchar.
+        """
 
         return fails_if(["oracle"],
                         'oracle converts empty strings to a blank space')
@@ -438,7 +489,9 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def unicode_connections(self):
-        """Target driver must support some encoding of Unicode across the wire."""
+        """
+        Target driver must support some encoding of Unicode across the wire.
+        """
         # TODO: expand to exclude MySQLdb versions w/ broken unicode
         return skip_if([
             exclude('mysql', '<', (4, 1, 1), 'no unicode connection support'),
@@ -475,16 +528,16 @@ class DefaultRequirements(SuiteRequirements):
             "driver doesn't support 'sane' rowcount"
         )
 
-
-
     @property
     def emulated_lastrowid(self):
         """"target dialect retrieves cursor.lastrowid or an equivalent
         after an insert() construct executes.
         """
         return fails_on_everything_except('mysql',
-                                      'sqlite+pysqlite', 'sqlite+pysqlcipher',
-                                      'sybase', 'mssql')
+                                          'sqlite+pysqlite',
+                                          'sqlite+pysqlcipher',
+                                          'sybase',
+                                          'mssql')
 
     @property
     def implements_get_lastrowid(self):
@@ -499,8 +552,9 @@ class DefaultRequirements(SuiteRequirements):
 
         """
         return skip_if('mssql+pymssql', 'crashes on pymssql') + \
-                    fails_on_everything_except('mysql',
-                                       'sqlite+pysqlite', 'sqlite+pysqlcipher')
+            fails_on_everything_except('mysql',
+                                       'sqlite+pysqlite',
+                                       'sqlite+pysqlcipher')
 
     @property
     def sane_multi_rowcount(self):
@@ -519,7 +573,26 @@ class DefaultRequirements(SuiteRequirements):
         """Target driver reflects the name of primary key constraints."""
 
         return fails_on_everything_except('postgresql', 'oracle', 'mssql',
-                    'sybase')
+                                          'sybase', 'sqlite')
+
+    @property
+    def array_type(self):
+        return only_on([
+            lambda config: against(config, "postgresql") and
+            not against(config, "+pg8000") and not against(config, "+zxjdbc")
+        ])
+
+    @property
+    def json_type(self):
+        return only_on([
+            lambda config: against(config, "mysql >= 5.7") and
+            not config.db.dialect._is_mariadb,
+            "postgresql >= 9.3"
+        ])
+
+    @property
+    def json_array_indexes(self):
+        return self.json_type + fails_if("+pg8000")
 
     @property
     def datetime_literals(self):
@@ -543,7 +616,7 @@ class DefaultRequirements(SuiteRequirements):
         datetime.datetime() with microsecond objects."""
 
         return skip_if(['mssql', 'mysql', 'firebird', '+zxjdbc',
-                    'oracle', 'sybase'])
+                        'oracle', 'sybase'])
 
     @property
     def datetime_historic(self):
@@ -586,8 +659,7 @@ class DefaultRequirements(SuiteRequirements):
         datetime.time() with microsecond objects."""
 
         return skip_if(['mssql', 'mysql', 'firebird', '+zxjdbc',
-                    'oracle', 'sybase'])
-
+                        'oracle', 'sybase'])
 
     @property
     def precision_numerics_general(self):
@@ -608,10 +680,9 @@ class DefaultRequirements(SuiteRequirements):
         to represent very large values."""
 
         return skip_if(
-                [("sybase+pyodbc", None, None,
-                    "Don't know how do get these values through FreeTDS + Sybase"),
-                ("firebird", None, None, "Precision must be from 1 to 18"),]
-            )
+            [("sybase+pyodbc", None, None,
+              "Don't know how do get these values through FreeTDS + Sybase"),
+             ("firebird", None, None, "Precision must be from 1 to 18")])
 
     @property
     def precision_numerics_many_significant_digits(self):
@@ -620,11 +691,12 @@ class DefaultRequirements(SuiteRequirements):
 
         """
         return fails_if(
-                    [('sqlite', None, None, 'TODO'),
-                    ("firebird", None, None, "Precision must be from 1 to 18"),
-                    ("sybase+pysybase", None, None, "TODO"),
-                    ('mssql+pymssql', None, None, 'FIXME: improve pymssql dec handling')]
-                )
+            [('sqlite', None, None, 'TODO'),
+             ("firebird", None, None, "Precision must be from 1 to 18"),
+             ("sybase+pysybase", None, None, "TODO"),
+             ('mssql+pymssql', None, None,
+              'FIXME: improve pymssql dec handling')]
+        )
 
     @property
     def precision_numerics_retains_significant_digits(self):
@@ -633,14 +705,14 @@ class DefaultRequirements(SuiteRequirements):
         the .000 maintained."""
 
         return fails_if(
-                [
-                    ('oracle', None, None,
-            "this may be a bug due to the difficulty in handling "
-            "oracle precision numerics"),
-                    ("firebird", None, None,
-        "database and/or driver truncates decimal places.")
-                ]
-                )
+            [
+                ('oracle', None, None,
+                 "this may be a bug due to the difficulty in handling "
+                 "oracle precision numerics"),
+                ("firebird", None, None,
+                 "database and/or driver truncates decimal places.")
+            ]
+        )
 
     @property
     def precision_generic_float_type(self):
@@ -648,35 +720,31 @@ class DefaultRequirements(SuiteRequirements):
         least seven decimal places when using the generic Float type."""
 
         return fails_if([
-                    ('mysql', None, None,
-                                'mysql FLOAT type only returns 4 decimals'),
-                    ('firebird', None, None,
-                                "firebird FLOAT type isn't high precision"),
-                ])
+            ('mysql', None, None,
+             'mysql FLOAT type only returns 4 decimals'),
+            ('firebird', None, None,
+             "firebird FLOAT type isn't high precision"),
+        ])
 
     @property
     def floats_to_four_decimals(self):
         return fails_if([
-                    ("mysql+oursql", None, None, "Floating point error"),
-                    ("firebird", None, None,
-                        "Firebird still has FP inaccuracy even "
-                        "with only four decimal places"),
-                    ('mssql+pyodbc', None, None,
-                                'mssql+pyodbc has FP inaccuracy even with '
-                                'only four decimal places '
-                            ),
-                    ('mssql+pymssql', None, None,
-                                'mssql+pymssql has FP inaccuracy even with '
-                                'only four decimal places '),
-                    (
-                        'postgresql+pg8000', None, None,
-                        'postgresql+pg8000 has FP inaccuracy even with '
-                        'only four decimal places '),
-                    (
-                        'postgresql+psycopg2cffi', None, None,
-                        'postgresql+psycopg2cffi has FP inaccuracy even with '
-                        'only four decimal places '),
-                ])
+            ("mysql+oursql", None, None, "Floating point error"),
+            ("firebird", None, None,
+             "Firebird still has FP inaccuracy even "
+             "with only four decimal places"),
+            ('mssql+pyodbc', None, None,
+             'mssql+pyodbc has FP inaccuracy even with '
+             'only four decimal places '),
+            ('mssql+pymssql', None, None,
+             'mssql+pymssql has FP inaccuracy even with '
+             'only four decimal places '),
+            ('postgresql+pg8000', None, None,
+             'postgresql+pg8000 has FP inaccuracy even with '
+             'only four decimal places '),
+            ('postgresql+psycopg2cffi', None, None,
+             'postgresql+psycopg2cffi has FP inaccuracy even with '
+             'only four decimal places ')])
 
     @property
     def fetch_null_from_numeric(self):
@@ -688,45 +756,6 @@ class DefaultRequirements(SuiteRequirements):
     def duplicate_key_raises_integrity_error(self):
         return fails_on("postgresql+pg8000")
 
-    @property
-    def python2(self):
-        return skip_if(
-                lambda: sys.version_info >= (3,),
-                "Python version 2.xx is required."
-                )
-
-    @property
-    def python3(self):
-        return skip_if(
-                lambda: sys.version_info < (3,),
-                "Python version 3.xx is required."
-                )
-
-    @property
-    def cpython(self):
-        return only_if(lambda: util.cpython,
-               "cPython interpreter needed"
-             )
-
-
-    @property
-    def non_broken_pickle(self):
-        from sqlalchemy.util import pickle
-        return only_if(
-            lambda: not util.pypy and pickle.__name__ == 'cPickle'
-                or sys.version_info >= (3, 2),
-            "Needs cPickle+cPython or newer Python 3 pickle"
-        )
-
-
-    @property
-    def predictable_gc(self):
-        """target platform must remove all cycles unconditionally when
-        gc.collect() is called, as well as clean out unreferenced subclasses.
-
-        """
-        return self.cpython
-
     @property
     def hstore(self):
         def check_hstore(config):
@@ -735,7 +764,7 @@ class DefaultRequirements(SuiteRequirements):
             try:
                 config.db.execute("SELECT 'a=>1,a=>2'::hstore;")
                 return True
-            except:
+            except Exception:
                 return False
 
         return only_if(check_hstore)
@@ -750,12 +779,11 @@ class DefaultRequirements(SuiteRequirements):
             try:
                 config.db.scalar("select '[1,2)'::int4range;")
                 return True
-            except:
+            except Exception:
                 return False
 
         return only_if(check_range_types)
 
-
     @property
     def oracle_test_dblink(self):
         return skip_if(
@@ -774,7 +802,7 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def postgresql_jsonb(self):
-        return skip_if(
+        return only_on("postgresql >= 9.4") + skip_if(
             lambda config:
             config.db.dialect.driver == "pg8000" and
             config.db.dialect._dbapi_version <= (1, 10, 1)
@@ -820,20 +848,19 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def order_by_label_with_expression(self):
         return fails_if([
-                    ('firebird', None, None, "kinterbasdb doesn't send full type information"),
-                    ('postgresql', None, None, 'only simple labels allowed'),
-                    ('sybase', None, None, 'only simple labels allowed'),
-                    ('mssql', None, None, 'only simple labels allowed')
-                ])
-
+            ('firebird', None, None,
+             "kinterbasdb doesn't send full type information"),
+            ('postgresql', None, None, 'only simple labels allowed'),
+            ('sybase', None, None, 'only simple labels allowed'),
+            ('mssql', None, None, 'only simple labels allowed')
+        ])
 
     @property
     def skip_mysql_on_windows(self):
         """Catchall for a large variety of MySQL on Windows failures"""
 
         return skip_if(self._has_mysql_on_windows,
-                "Not supported on MySQL + Windows"
-            )
+                       "Not supported on MySQL + Windows")
 
     @property
     def mssql_freetds(self):
@@ -847,6 +874,10 @@ class DefaultRequirements(SuiteRequirements):
             )
         )
 
+    @property
+    def ad_hoc_engines(self):
+        return exclusions.skip_if(["oracle"])
+
     @property
     def no_mssql_freetds(self):
         return self.mssql_freetds.not_()
@@ -854,7 +885,8 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
-        return skip_if(["oracle", "firebird"], "non-standard SELECT scalar syntax")
+        return skip_if(["oracle", "firebird"],
+                       "non-standard SELECT scalar syntax")
 
     @property
     def mysql_fsp(self):
@@ -867,16 +899,16 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def mysql_zero_date(self):
         def check(config):
-             row = config.db.execute("show variables like 'sql_mode'").first()
-             return not row or "NO_ZERO_DATE" not in row[1]
+            row = config.db.execute("show variables like 'sql_mode'").first()
+            return not row or "NO_ZERO_DATE" not in row[1]
 
         return only_if(check)
 
     @property
     def mysql_non_strict(self):
         def check(config):
-             row = config.db.execute("show variables like 'sql_mode'").first()
-             return not row or "STRICT" not in row[1]
+            row = config.db.execute("show variables like 'sql_mode'").first()
+            return not row or "STRICT" not in row[1]
 
         return only_if(check)
 
