@@ -1,5 +1,5 @@
 from sqlalchemy.testing import eq_, assert_raises_message, \
-    assert_raises, AssertsCompiledSQL
+    assert_raises, AssertsCompiledSQL, expect_warnings
 import datetime
 from sqlalchemy.schema import CreateSequence, DropSequence, CreateTable
 from sqlalchemy.sql import select, text, literal_column
@@ -24,6 +24,9 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = 'default'
 
     def test_string(self):
+        # note: that the datatype is an Integer here doesn't matter,
+        # the server_default is interpreted independently of the
+        # column's datatype.
         m = MetaData()
         t = Table('t', m, Column('x', Integer, server_default='5'))
         self.assert_compile(
@@ -31,6 +34,14 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
             "CREATE TABLE t (x INTEGER DEFAULT '5')"
         )
 
+    def test_string_w_quotes(self):
+        m = MetaData()
+        t = Table('t', m, Column('x', Integer, server_default="5'6"))
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE t (x INTEGER DEFAULT '5''6')"
+        )
+
     def test_text(self):
         m = MetaData()
         t = Table('t', m, Column('x', Integer, server_default=text('5 + 8')))
@@ -39,6 +50,23 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
             "CREATE TABLE t (x INTEGER DEFAULT 5 + 8)"
         )
 
+    def test_text_w_quotes(self):
+        m = MetaData()
+        t = Table('t', m, Column('x', Integer, server_default=text("5 ' 8")))
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE t (x INTEGER DEFAULT 5 ' 8)"
+        )
+
+    def test_literal_binds_w_quotes(self):
+        m = MetaData()
+        t = Table('t', m, Column('x', Integer,
+                  server_default=literal("5 ' 8")))
+        self.assert_compile(
+            CreateTable(t),
+            """CREATE TABLE t (x INTEGER DEFAULT '5 '' 8')"""
+        )
+
     def test_text_literal_binds(self):
         m = MetaData()
         t = Table(
@@ -398,9 +426,9 @@ class DefaultTest(fixtures.TestBase):
 
         ctexec = sa.select(
             [currenttime.label('now')], bind=testing.db).scalar()
-        l = t.select().order_by(t.c.col1).execute()
+        result = t.select().order_by(t.c.col1).execute()
         today = datetime.date.today()
-        eq_(l.fetchall(), [
+        eq_(result.fetchall(), [
             (x, 'imthedefault', f, ts, ts, ctexec, True, False,
              12, today, 'py', 'hi', 'BINDfoo')
             for x in range(51, 54)])
@@ -418,9 +446,9 @@ class DefaultTest(fixtures.TestBase):
         t.insert().execute({}, {}, {})
 
         ctexec = currenttime.scalar()
-        l = t.select().execute()
+        result = t.select().execute()
         today = datetime.date.today()
-        eq_(l.fetchall(),
+        eq_(result.fetchall(),
             [(51, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, 'py', 'hi', 'BINDfoo'),
              (52, 'imthedefault', f, ts, ts, ctexec, True, False,
@@ -434,9 +462,9 @@ class DefaultTest(fixtures.TestBase):
         t.insert().values([{}, {}, {}]).execute()
 
         ctexec = currenttime.scalar()
-        l = t.select().execute()
+        result = t.select().execute()
         today = datetime.date.today()
-        eq_(l.fetchall(),
+        eq_(result.fetchall(),
             [(51, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, 'py', 'hi', 'BINDfoo'),
              (52, 'imthedefault', f, ts, ts, ctexec, True, False,
@@ -484,8 +512,8 @@ class DefaultTest(fixtures.TestBase):
 
     def test_insert_values(self):
         t.insert(values={'col3': 50}).execute()
-        l = t.select().execute()
-        eq_(50, l.first()['col3'])
+        result = t.select().execute()
+        eq_(50, result.first()['col3'])
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_updatemany(self):
@@ -504,10 +532,10 @@ class DefaultTest(fixtures.TestBase):
             {'pkval': 52},
             {'pkval': 53})
 
-        l = t.select().execute()
+        result = t.select().execute()
         ctexec = currenttime.scalar()
         today = datetime.date.today()
-        eq_(l.fetchall(),
+        eq_(result.fetchall(),
             [(51, 'im the update', f2, ts, ts, ctexec, False, False,
               13, today, 'py', 'hi', 'BINDfoo'),
              (52, 'im the update', f2, ts, ts, ctexec, True, False,
@@ -521,9 +549,9 @@ class DefaultTest(fixtures.TestBase):
         pk = r.inserted_primary_key[0]
         t.update(t.c.col1 == pk).execute(col4=None, col5=None)
         ctexec = currenttime.scalar()
-        l = t.select(t.c.col1 == pk).execute()
-        l = l.first()
-        eq_(l,
+        result = t.select(t.c.col1 == pk).execute()
+        result = result.first()
+        eq_(result,
             (pk, 'im the update', f2, None, None, ctexec, True, False,
              13, datetime.date.today(), 'py', 'hi', 'BINDfoo'))
         eq_(11, f2)
@@ -533,9 +561,96 @@ class DefaultTest(fixtures.TestBase):
         r = t.insert().execute()
         pk = r.inserted_primary_key[0]
         t.update(t.c.col1 == pk, values={'col3': 55}).execute()
-        l = t.select(t.c.col1 == pk).execute()
-        l = l.first()
-        eq_(55, l['col3'])
+        result = t.select(t.c.col1 == pk).execute()
+        result = result.first()
+        eq_(55, result['col3'])
+
+
+class CTEDefaultTest(fixtures.TablesTest):
+    __requires__ = ('ctes',)
+    __backend__ = True
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table(
+            'q', metadata,
+            Column('x', Integer, default=2),
+            Column('y', Integer, onupdate=5),
+            Column('z', Integer)
+        )
+
+        Table(
+            'p', metadata,
+            Column('s', Integer),
+            Column('t', Integer),
+            Column('u', Integer, onupdate=1)
+        )
+
+    def _test_a_in_b(self, a, b):
+        q = self.tables.q
+        p = self.tables.p
+
+        with testing.db.connect() as conn:
+            if a == 'delete':
+                conn.execute(q.insert().values(y=10, z=1))
+                cte = q.delete().\
+                    where(q.c.z == 1).returning(q.c.z).cte('c')
+                expected = None
+            elif a == "insert":
+                cte = q.insert().values(z=1, y=10).returning(q.c.z).cte('c')
+                expected = (2, 10)
+            elif a == "update":
+                conn.execute(q.insert().values(x=5, y=10, z=1))
+                cte = q.update().\
+                    where(q.c.z == 1).values(x=7).returning(q.c.z).cte('c')
+                expected = (7, 5)
+            elif a == "select":
+                conn.execute(q.insert().values(x=5, y=10, z=1))
+                cte = sa.select([q.c.z]).cte('c')
+                expected = (5, 10)
+
+            if b == "select":
+                conn.execute(p.insert().values(s=1))
+                stmt = select([p.c.s, cte.c.z])
+            elif b == "insert":
+                sel = select([1, cte.c.z, ])
+                stmt = p.insert().from_select(['s', 't'], sel).returning(
+                    p.c.s, p.c.t)
+            elif b == "delete":
+                stmt = p.insert().values(s=1, t=cte.c.z).returning(
+                    p.c.s, cte.c.z)
+            elif b == "update":
+                conn.execute(p.insert().values(s=1))
+                stmt = p.update().values(t=5).\
+                    where(p.c.s == cte.c.z).\
+                    returning(p.c.u, cte.c.z)
+            eq_(
+                conn.execute(stmt).fetchall(),
+                [(1, 1)]
+            )
+
+            eq_(
+                conn.execute(select([q.c.x, q.c.y])).fetchone(),
+                expected
+            )
+
+    def test_update_in_select(self):
+        self._test_a_in_b("update", "select")
+
+    def test_delete_in_select(self):
+        self._test_a_in_b("update", "select")
+
+    def test_insert_in_select(self):
+        self._test_a_in_b("update", "select")
+
+    def test_select_in_update(self):
+        self._test_a_in_b("select", "update")
+
+    def test_select_in_insert(self):
+        self._test_a_in_b("select", "insert")
+
+    # TODO: updates / inserts can be run in one statement w/ CTE ?
+    # deletes?
 
 
 class PKDefaultTest(fixtures.TablesTest):
@@ -635,10 +750,10 @@ class PKIncrementTest(fixtures.TablesTest):
         try:
             try:
                 self._test_autoincrement(con)
-            except:
+            except Exception:
                 try:
                     tx.rollback()
-                except:
+                except Exception:
                     pass
                 raise
             else:
@@ -732,7 +847,7 @@ class AutoIncrementTest(fixtures.TablesTest):
         )
         assert x._autoincrement_column is None
 
-    @testing.fails_on('sqlite', 'FIXME: unknown')
+    @testing.only_on("sqlite")
     def test_non_autoincrement(self):
         # sqlite INT primary keys can be non-unique! (only for ints)
         nonai = Table(
@@ -746,12 +861,12 @@ class AutoIncrementTest(fixtures.TablesTest):
             # mysql in legacy mode fails on second row
             nonai.insert().execute(data='row 1')
             nonai.insert().execute(data='row 2')
-        assert_raises(
-            sa.exc.DBAPIError,
-            go
-        )
 
-        nonai.insert().execute(id=1, data='row 1')
+        # just testing SQLite for now, it passes
+        with expect_warnings(
+            ".*has no Python-side or server-side default.*",
+        ):
+            go()
 
     def test_col_w_sequence_non_autoinc_no_firing(self):
         metadata = self.metadata
@@ -1081,6 +1196,24 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         assert not self._has_sequence('s1')
         assert not self._has_sequence('s2')
 
+    @testing.requires.returning
+    @testing.provide_metadata
+    def test_freestanding_sequence_via_autoinc(self):
+        t = Table(
+            'some_table', self.metadata,
+            Column(
+                'id', Integer,
+                autoincrement=True,
+                primary_key=True,
+                default=Sequence(
+                    'my_sequence', metadata=self.metadata).next_value())
+        )
+        self.metadata.create_all(testing.db)
+
+        result = testing.db.execute(t.insert())
+        eq_(result.inserted_primary_key, [1])
+
+
 cartitems = sometable = metadata = None
 
 
