@@ -155,15 +155,19 @@ class SelectableTest(
         assert c in s.c.bar.proxy_set
 
     def test_no_error_on_unsupported_expr_key(self):
-        from sqlalchemy.dialects.postgresql import ARRAY
+        from sqlalchemy.sql.expression import BinaryExpression
 
-        t = table('t', column('x', ARRAY(Integer)))
+        def myop(x, y):
+            pass
+
+        t = table('t', column('x'), column('y'))
+
+        expr = BinaryExpression(t.c.x, t.c.y, myop)
 
-        expr = t.c.x[5]
         s = select([t, expr])
         eq_(
             s.c.keys(),
-            ['x', expr.anon_label]
+            ['x', 'y', expr.anon_label]
         )
 
     def test_cloned_intersection(self):
@@ -370,8 +374,8 @@ class SelectableTest(
         assert u1.corresponding_column(table1.c.col3) is u1.c.col1
 
     def test_singular_union(self):
-        u = union(select([table1.c.col1, table1.c.col2, table1.c.col3]), select(
-            [table1.c.col1, table1.c.col2, table1.c.col3]))
+        u = union(select([table1.c.col1, table1.c.col2, table1.c.col3]),
+                  select([table1.c.col1, table1.c.col2, table1.c.col3]))
         u = union(select([table1.c.col1, table1.c.col2, table1.c.col3]))
         assert u.c.col1 is not None
         assert u.c.col2 is not None
@@ -385,11 +389,10 @@ class SelectableTest(
                     table1.c.col2,
                     table1.c.col3,
                     table1.c.colx,
-                    null().label('coly')]).union(select([table2.c.col1,
-                                                         table2.c.col2,
-                                                         table2.c.col3,
-                                                         null().label('colx'),
-                                                         table2.c.coly])).alias('analias')
+                    null().label('coly')]).union(
+                        select([table2.c.col1, table2.c.col2, table2.c.col3,
+                                null().label('colx'), table2.c.coly])
+                    ).alias('analias')
         s1 = table1.select(use_labels=True)
         s2 = table2.select(use_labels=True)
         assert u.corresponding_column(s1.c.table1_col2) is u.c.col2
@@ -458,6 +461,26 @@ class SelectableTest(
         assert u1.corresponding_column(table2.c.col1) is u1.c._all_columns[0]
         assert u1.corresponding_column(table2.c.col3) is u1.c._all_columns[2]
 
+    @testing.emits_warning("Column 'col1'")
+    def test_union_alias_dupe_keys_grouped(self):
+        s1 = select([table1.c.col1, table1.c.col2, table2.c.col1]).\
+            limit(1).alias()
+        s2 = select([table2.c.col1, table2.c.col2, table2.c.col3]).limit(1)
+        u1 = union(s1, s2)
+
+        assert u1.corresponding_column(
+            s1.c._all_columns[0]) is u1.c._all_columns[0]
+        assert u1.corresponding_column(s2.c.col1) is u1.c._all_columns[0]
+        assert u1.corresponding_column(s1.c.col2) is u1.c.col2
+        assert u1.corresponding_column(s2.c.col2) is u1.c.col2
+
+        assert u1.corresponding_column(s2.c.col3) is u1.c._all_columns[2]
+
+        # this differs from the non-alias test because table2.c.col1 is
+        # more directly at s2.c.col1 than it is s1.c.col1.
+        assert u1.corresponding_column(table2.c.col1) is u1.c._all_columns[0]
+        assert u1.corresponding_column(table2.c.col3) is u1.c._all_columns[2]
+
     def test_select_union(self):
 
         # like testaliasunion, but off a Select off the union.
@@ -466,11 +489,10 @@ class SelectableTest(
                     table1.c.col2,
                     table1.c.col3,
                     table1.c.colx,
-                    null().label('coly')]).union(select([table2.c.col1,
-                                                         table2.c.col2,
-                                                         table2.c.col3,
-                                                         null().label('colx'),
-                                                         table2.c.coly])).alias('analias')
+                    null().label('coly')]).union(
+                        select([table2.c.col1, table2.c.col2, table2.c.col3,
+                                null().label('colx'), table2.c.coly])
+                    ).alias('analias')
         s = select([u])
         s1 = table1.select(use_labels=True)
         s2 = table2.select(use_labels=True)
@@ -485,11 +507,10 @@ class SelectableTest(
                     table1.c.col2,
                     table1.c.col3,
                     table1.c.colx,
-                    null().label('coly')]).union(select([table2.c.col1,
-                                                         table2.c.col2,
-                                                         table2.c.col3,
-                                                         null().label('colx'),
-                                                         table2.c.coly])).alias('analias')
+                    null().label('coly')]).union(
+                        select([table2.c.col1, table2.c.col2, table2.c.col3,
+                                null().label('colx'), table2.c.coly])
+                    ).alias('analias')
         j1 = table1.join(table2)
         assert u.corresponding_column(j1.c.table1_colx) is u.c.colx
         assert j1.corresponding_column(u.c.colx) is j1.c.table1_colx
@@ -589,7 +610,8 @@ class SelectableTest(
         s2 = select([s.label('c')])
         self.assert_compile(
             s2.select(),
-            "SELECT c FROM (SELECT (SELECT (SELECT table1.col1 AS a FROM table1) AS b) AS c)"
+            "SELECT c FROM (SELECT (SELECT ("
+            "SELECT table1.col1 AS a FROM table1) AS b) AS c)"
         )
 
     def test_self_referential_select_raises(self):
@@ -892,6 +914,44 @@ class RefreshForNewColTest(fixtures.TestBase):
         j._refresh_for_new_column(q)
         assert j.c.b_q is q
 
+    def test_fk_table(self):
+        m = MetaData()
+        fk = ForeignKey('x.id')
+        Table('x', m, Column('id', Integer))
+        a = Table('a', m, Column('x', Integer, fk))
+        a.c
+
+        q = Column('q', Integer)
+        a.append_column(q)
+        a._refresh_for_new_column(q)
+        eq_(a.foreign_keys, set([fk]))
+
+        fk2 = ForeignKey('g.id')
+        p = Column('p', Integer, fk2)
+        a.append_column(p)
+        a._refresh_for_new_column(p)
+        eq_(a.foreign_keys, set([fk, fk2]))
+
+    def test_fk_join(self):
+        m = MetaData()
+        fk = ForeignKey('x.id')
+        Table('x', m, Column('id', Integer))
+        a = Table('a', m, Column('x', Integer, fk))
+        b = Table('b', m, Column('y', Integer))
+        j = a.join(b, a.c.x == b.c.y)
+        j.c
+
+        q = Column('q', Integer)
+        b.append_column(q)
+        j._refresh_for_new_column(q)
+        eq_(j.foreign_keys, set([fk]))
+
+        fk2 = ForeignKey('g.id')
+        p = Column('p', Integer, fk2)
+        b.append_column(p)
+        j._refresh_for_new_column(p)
+        eq_(j.foreign_keys, set([fk, fk2]))
+
 
 class AnonLabelTest(fixtures.TestBase):
 
@@ -912,10 +972,10 @@ class AnonLabelTest(fixtures.TestBase):
         c1 = func.count('*')
         assert c1.label(None) is not c1
 
-        eq_(str(select([c1])), "SELECT count(:param_1) AS count_1")
+        eq_(str(select([c1])), "SELECT count(:count_2) AS count_1")
         c2 = select([c1]).compile()
 
-        eq_(str(select([c1.label(None)])), "SELECT count(:param_1) AS count_1")
+        eq_(str(select([c1.label(None)])), "SELECT count(:count_2) AS count_1")
 
     def test_named_labels_named_column(self):
         c1 = column('x')
@@ -1402,17 +1462,18 @@ class ReduceTest(fixtures.TestBase, AssertsExecutionResults):
             Column('primary_language', String(50)),
         )
         managers = Table(
-            'managers', metadata, Column(
-                'person_id', Integer, ForeignKey('people.person_id'), primary_key=True), Column(
-                'status', String(30)), Column(
-                'manager_name', String(50)))
+            'managers', metadata,
+            Column('person_id', Integer, ForeignKey('people.person_id'),
+                   primary_key=True),
+            Column('status', String(30)),
+            Column('manager_name', String(50)))
         pjoin = \
             people.outerjoin(engineers).outerjoin(managers).\
             select(use_labels=True).alias('pjoin'
                                           )
-        eq_(util.column_set(sql_util.reduce_columns([pjoin.c.people_person_id,
-                                                     pjoin.c.engineers_person_id,
-                                                     pjoin.c.managers_person_id])),
+        eq_(util.column_set(sql_util.reduce_columns(
+            [pjoin.c.people_person_id, pjoin.c.engineers_person_id,
+             pjoin.c.managers_person_id])),
             util.column_set([pjoin.c.people_person_id]))
 
     def test_reduce_aliased_union(self):
@@ -1491,7 +1552,8 @@ class ReduceTest(fixtures.TestBase, AssertsExecutionResults):
             select_from(page_table.join(magazine_page_table))
         ).alias('pjoin')
         eq_(util.column_set(sql_util.reduce_columns(
-            [pjoin.c.id, pjoin.c.page_id, pjoin.c.magazine_page_id])), util.column_set([pjoin.c.id]))
+            [pjoin.c.id, pjoin.c.page_id, pjoin.c.magazine_page_id])),
+            util.column_set([pjoin.c.id]))
 
         # the first selectable has a CAST, which is a placeholder for
         # classified_page.magazine_page_id in the second selectable.
@@ -1516,7 +1578,8 @@ class ReduceTest(fixtures.TestBase, AssertsExecutionResults):
                         join(classified_page_table))
         ).alias('pjoin')
         eq_(util.column_set(sql_util.reduce_columns(
-            [pjoin.c.id, pjoin.c.page_id, pjoin.c.magazine_page_id])), util.column_set([pjoin.c.id]))
+            [pjoin.c.id, pjoin.c.page_id, pjoin.c.magazine_page_id])),
+            util.column_set([pjoin.c.id]))
 
 
 class DerivedTest(fixtures.TestBase, AssertsExecutionResults):
@@ -1771,10 +1834,11 @@ class AnnotationsTest(fixtures.TestBase):
             assert elem == {}
 
         assert b2.left is not bin.left
-        assert b3.left is not b2.left is not bin.left
+        assert b3.left is not b2.left and b2.left is not bin.left
         assert b4.left is bin.left  # since column is immutable
         # deannotate copies the element
-        assert bin.right is not b2.right is not b3.right is not b4.right
+        assert bin.right is not b2.right and b2.right is not b3.right \
+            and b3.right is not b4.right
 
     def test_annotate_unique_traversal(self):
         """test that items are copied only once during
@@ -2240,6 +2304,7 @@ class ResultMapTest(fixtures.TestBase):
             [Boolean]
         )
 
+
 class ForUpdateTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
