@@ -1,5 +1,6 @@
 # coding: utf-8
-from sqlalchemy.testing import eq_, assert_raises, assert_raises_message, expect_warnings
+from sqlalchemy.testing import eq_, is_, is_not_, assert_raises, \
+    assert_raises_message, expect_warnings
 import decimal
 import datetime
 import os
@@ -9,13 +10,15 @@ from sqlalchemy import (
     and_, func, Date, LargeBinary, literal, cast, text, Enum,
     type_coerce, VARCHAR, Time, DateTime, BigInteger, SmallInteger, BOOLEAN,
     BLOB, NCHAR, NVARCHAR, CLOB, TIME, DATE, DATETIME, TIMESTAMP, SMALLINT,
-    INTEGER, DECIMAL, NUMERIC, FLOAT, REAL)
+    INTEGER, DECIMAL, NUMERIC, FLOAT, REAL, ARRAY, JSON)
 from sqlalchemy.sql import ddl
-
+from sqlalchemy.sql import visitors
+from sqlalchemy import inspection
 from sqlalchemy import exc, types, util, dialects
-for name in dialects.__all__:
+from sqlalchemy.util import OrderedDict
+for name in dialects.__all__:  # noqa
     __import__("sqlalchemy.dialects.%s" % name)
-from sqlalchemy.sql import operators, column, table
+from sqlalchemy.sql import operators, column, table, null
 from sqlalchemy.schema import CheckConstraint, AddConstraint
 from sqlalchemy.engine import default
 from sqlalchemy.testing.schema import Table, Column
@@ -25,6 +28,7 @@ from sqlalchemy.testing import AssertsCompiledSQL, AssertsExecutionResults, \
 from sqlalchemy.testing.util import picklers
 from sqlalchemy.testing.util import round_decimal
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import mock
 
 
 class AdaptTest(fixtures.TestBase):
@@ -137,7 +141,7 @@ class AdaptTest(fixtures.TestBase):
         for is_down_adaption, typ, target_adaptions in adaptions():
             if typ in (types.TypeDecorator, types.TypeEngine, types.Variant):
                 continue
-            elif typ is dialects.postgresql.ARRAY:
+            elif issubclass(typ, ARRAY):
                 t1 = typ(String)
             else:
                 t1 = typ()
@@ -145,6 +149,8 @@ class AdaptTest(fixtures.TestBase):
                 if not issubclass(typ, types.Enum) and \
                         issubclass(cls, types.Enum):
                     continue
+                if cls.__module__.startswith("test"):
+                    continue
 
                 # print("ADAPT %s -> %s" % (t1.__class__, cls))
                 t2 = t1.adapt(cls)
@@ -176,6 +182,7 @@ class AdaptTest(fixtures.TestBase):
         eq_(types.String().python_type, str)
         eq_(types.Unicode().python_type, util.text_type)
         eq_(types.String(convert_unicode=True).python_type, util.text_type)
+        eq_(types.Enum('one', 'two', 'three').python_type, str)
 
         assert_raises(
             NotImplementedError,
@@ -187,12 +194,28 @@ class AdaptTest(fixtures.TestBase):
         for typ in self._all_types():
             if typ in (types.TypeDecorator, types.TypeEngine, types.Variant):
                 continue
-            elif typ is dialects.postgresql.ARRAY:
+            elif issubclass(typ, ARRAY):
                 t1 = typ(String)
             else:
                 t1 = typ()
             repr(t1)
 
+    def test_adapt_constructor_copy_override_kw(self):
+        """test that adapt() can accept kw args that override
+        the state of the original object.
+
+        This essentially is testing the behavior of util.constructor_copy().
+
+        """
+        t1 = String(length=50, convert_unicode=False)
+        t2 = t1.adapt(Text, convert_unicode=True)
+        eq_(
+            t2.length, 50
+        )
+        eq_(
+            t2.convert_unicode, True
+        )
+
 
 class TypeAffinityTest(fixtures.TestBase):
 
@@ -256,7 +279,6 @@ class PickleTypesTest(fixtures.TestBase):
                 Column('Lar', LargeBinary()),
                 Column('Pic', PickleType()),
                 Column('Int', Interval()),
-                Column('Enu', Enum('x', 'y', 'z', name="somename")),
             ]
             for column_type in column_types:
                 meta = MetaData()
@@ -281,14 +303,14 @@ class UserDefinedTest(fixtures.TablesTest, AssertsCompiledSQL):
             user_id=4, goofy='fred', goofy2='fred', goofy4=util.u('fred'),
             goofy7=util.u('fred'), goofy8=9, goofy9=9)
 
-        l = users.select().order_by(users.c.user_id).execute().fetchall()
+        result = users.select().order_by(users.c.user_id).execute().fetchall()
         for assertstr, assertint, assertint2, row in zip(
             [
                 "BIND_INjackBIND_OUT", "BIND_INlalaBIND_OUT",
                 "BIND_INfredBIND_OUT"],
             [1200, 1500, 900],
             [1800, 2250, 1350],
-            l
+            result
         ):
             for col in list(row)[1:5]:
                 eq_(col, assertstr)
@@ -771,6 +793,68 @@ class TypeCoerceCastTest(fixtures.TablesTest):
             [('BIND_INd1', 'BIND_INd1BIND_OUT')]
         )
 
+    def test_cast_replace_col_w_bind(self):
+        self._test_replace_col_w_bind(cast)
+
+    def test_type_coerce_replace_col_w_bind(self):
+        self._test_replace_col_w_bind(type_coerce)
+
+    def _test_replace_col_w_bind(self, coerce_fn):
+        MyType = self.MyType
+
+        t = self.tables.t
+        t.insert().values(data=coerce_fn('d1', MyType)).execute()
+
+        stmt = select([t.c.data, coerce_fn(t.c.data, MyType)])
+
+        def col_to_bind(col):
+            if col is t.c.data:
+                return bindparam(None, "x", type_=col.type, unique=True)
+            return None
+
+        # ensure we evaulate the expression so that we can see
+        # the clone resets this info
+        stmt.compile()
+
+        new_stmt = visitors.replacement_traverse(stmt, {}, col_to_bind)
+
+        # original statement
+        eq_(
+            testing.db.execute(stmt).fetchall(),
+            [('BIND_INd1', 'BIND_INd1BIND_OUT')]
+        )
+
+        # replaced with binds; CAST can't affect the bound parameter
+        # on the way in here
+        eq_(
+            testing.db.execute(new_stmt).fetchall(),
+            [('x', 'BIND_INxBIND_OUT')] if coerce_fn is type_coerce
+            else [('x', 'xBIND_OUT')]
+        )
+
+    def test_cast_bind(self):
+        self._test_bind(cast)
+
+    def test_type_bind(self):
+        self._test_bind(type_coerce)
+
+    def _test_bind(self, coerce_fn):
+        MyType = self.MyType
+
+        t = self.tables.t
+        t.insert().values(data=coerce_fn('d1', MyType)).execute()
+
+        stmt = select([
+            bindparam(None, "x", String(50), unique=True),
+            coerce_fn(bindparam(None, "x", String(50), unique=True), MyType)
+        ])
+
+        eq_(
+            testing.db.execute(stmt).fetchall(),
+            [('x', 'BIND_INxBIND_OUT')] if coerce_fn is type_coerce
+            else [('x', 'xBIND_OUT')]
+        )
+
     @testing.fails_on(
         "oracle", "ORA-00906: missing left parenthesis - "
         "seems to be CAST(:param AS type)")
@@ -922,6 +1006,55 @@ class VariantTest(fixtures.TestBase, AssertsCompiledSQL):
             'fooUTWO'
         )
 
+    def test_comparator_variant(self):
+        expr = column('x', self.variant) == "bar"
+        is_(
+            expr.right.type, self.variant
+        )
+
+    @testing.only_on("sqlite")
+    @testing.provide_metadata
+    def test_round_trip(self):
+        variant = self.UTypeOne().with_variant(
+            self.UTypeTwo(), 'sqlite')
+
+        t = Table('t', self.metadata,
+                Column('x', variant)
+        )
+        with testing.db.connect() as conn:
+            t.create(conn)
+
+            conn.execute(
+                t.insert(),
+                x='foo'
+            )
+
+            eq_(
+                conn.scalar(select([t.c.x]).where(t.c.x == 'foo')),
+                'fooUTWO'
+            )
+
+    @testing.only_on("sqlite")
+    @testing.provide_metadata
+    def test_round_trip_sqlite_datetime(self):
+        variant = DateTime().with_variant(
+            dialects.sqlite.DATETIME(truncate_microseconds=True), 'sqlite')
+
+        t = Table('t', self.metadata,
+                Column('x', variant)
+        )
+        with testing.db.connect() as conn:
+            t.create(conn)
+
+            conn.execute(
+                t.insert(),
+                x=datetime.datetime(2015, 4, 18, 10, 15, 17, 4839)
+            )
+
+            eq_(
+                conn.scalar(select([t.c.x]).where(t.c.x == datetime.datetime(2015, 4, 18, 10, 15, 17, 1059))),
+                datetime.datetime(2015, 4, 18, 10, 15, 17)
+            )
 
 class UnicodeTest(fixtures.TestBase):
 
@@ -1003,41 +1136,206 @@ class UnicodeTest(fixtures.TestBase):
             unicodedata.encode('ascii', 'ignore').decode()
         )
 
-enum_table = non_native_enum_table = metadata = None
 
+class EnumTest(AssertsCompiledSQL, fixtures.TablesTest):
+    __backend__ = True
 
-class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
+    class SomeEnum(object):
+        # Implements PEP 435 in the minimal fashion needed by SQLAlchemy
+        __members__ = OrderedDict()
+
+        def __init__(self, name, value):
+            self.name = name
+            self.value = value
+            self.__members__[name] = self
+            setattr(self.__class__, name, self)
+
+    one = SomeEnum('one', 1)
+    two = SomeEnum('two', 2)
+    three = SomeEnum('three', 3)
 
     @classmethod
-    def setup_class(cls):
-        global enum_table, non_native_enum_table, metadata
-        metadata = MetaData(testing.db)
-        enum_table = Table(
+    def define_tables(cls, metadata):
+        Table(
             'enum_table', metadata, Column("id", Integer, primary_key=True),
             Column('someenum', Enum('one', 'two', 'three', name='myenum'))
         )
 
-        non_native_enum_table = Table(
+        Table(
             'non_native_enum_table', metadata,
             Column("id", Integer, primary_key=True),
             Column('someenum', Enum('one', 'two', 'three', native_enum=False)),
+            Column('someotherenum',
+                   Enum('one', 'two', 'three',
+                        create_constraint=False, native_enum=False,
+                        validate_strings=True)),
         )
 
-        metadata.create_all()
+        Table(
+            'stdlib_enum_table', metadata,
+            Column("id", Integer, primary_key=True),
+            Column('someenum', Enum(cls.SomeEnum))
+        )
 
-    def teardown(self):
-        enum_table.delete().execute()
-        non_native_enum_table.delete().execute()
+    def test_python_type(self):
+        eq_(types.Enum(self.SomeEnum).python_type, self.SomeEnum)
 
-    @classmethod
-    def teardown_class(cls):
-        metadata.drop_all()
+    def test_pickle_types(self):
+        global SomeEnum
+        SomeEnum = self.SomeEnum
+        for loads, dumps in picklers():
+            column_types = [
+                Column('Enu', Enum('x', 'y', 'z', name="somename")),
+                Column('En2', Enum(self.SomeEnum)),
+            ]
+            for column_type in column_types:
+                meta = MetaData()
+                Table('foo', meta, column_type)
+                loads(dumps(column_type))
+                loads(dumps(meta))
+
+    def test_validators_pep435(self):
+        type_ = Enum(self.SomeEnum)
+        validate_type = Enum(self.SomeEnum, validate_strings=True)
+
+        bind_processor = type_.bind_processor(testing.db.dialect)
+        bind_processor_validates = validate_type.bind_processor(
+            testing.db.dialect)
+        eq_(bind_processor('one'), "one")
+        eq_(bind_processor(self.one), "one")
+        eq_(bind_processor("foo"), "foo")
+        assert_raises_message(
+            LookupError,
+            '"5" is not among the defined enum values',
+            bind_processor, 5
+        )
+
+        assert_raises_message(
+            LookupError,
+            '"foo" is not among the defined enum values',
+            bind_processor_validates, "foo"
+        )
+
+        result_processor = type_.result_processor(testing.db.dialect, None)
+
+        eq_(result_processor('one'), self.one)
+        assert_raises_message(
+            LookupError,
+            '"foo" is not among the defined enum values',
+            result_processor, "foo"
+        )
+
+        literal_processor = type_.literal_processor(testing.db.dialect)
+        validate_literal_processor = validate_type.literal_processor(
+            testing.db.dialect)
+        eq_(literal_processor("one"), "'one'")
+
+        eq_(literal_processor("foo"), "'foo'")
+
+        assert_raises_message(
+            LookupError,
+            '"5" is not among the defined enum values',
+            literal_processor, 5
+        )
+
+        assert_raises_message(
+            LookupError,
+            '"foo" is not among the defined enum values',
+            validate_literal_processor, "foo"
+        )
+
+    def test_validators_plain(self):
+        type_ = Enum("one", "two")
+        validate_type = Enum("one", "two", validate_strings=True)
+
+        bind_processor = type_.bind_processor(testing.db.dialect)
+        bind_processor_validates = validate_type.bind_processor(
+            testing.db.dialect)
+        eq_(bind_processor('one'), "one")
+        eq_(bind_processor('foo'), "foo")
+        assert_raises_message(
+            LookupError,
+            '"5" is not among the defined enum values',
+            bind_processor, 5
+        )
+
+        assert_raises_message(
+            LookupError,
+            '"foo" is not among the defined enum values',
+            bind_processor_validates, "foo"
+        )
+
+        result_processor = type_.result_processor(testing.db.dialect, None)
+
+        eq_(result_processor('one'), "one")
+        assert_raises_message(
+            LookupError,
+            '"foo" is not among the defined enum values',
+            result_processor, "foo"
+        )
+
+        literal_processor = type_.literal_processor(testing.db.dialect)
+        validate_literal_processor = validate_type.literal_processor(
+            testing.db.dialect)
+        eq_(literal_processor("one"), "'one'")
+        eq_(literal_processor("foo"), "'foo'")
+        assert_raises_message(
+            LookupError,
+            '"5" is not among the defined enum values',
+            literal_processor, 5
+        )
+
+        assert_raises_message(
+            LookupError,
+            '"foo" is not among the defined enum values',
+            validate_literal_processor, "foo"
+        )
+
+    def test_validators_not_in_like_roundtrip(self):
+        enum_table = self.tables['non_native_enum_table']
+
+        enum_table.insert().execute([
+            {'id': 1, 'someenum': 'two'},
+            {'id': 2, 'someenum': 'two'},
+            {'id': 3, 'someenum': 'one'},
+        ])
+
+        eq_(
+            enum_table.select().
+            where(enum_table.c.someenum.like('%wo%')).
+            order_by(enum_table.c.id).execute().fetchall(),
+            [
+                (1, 'two', None),
+                (2, 'two', None),
+            ]
+        )
+
+    def test_validators_not_in_concatenate_roundtrip(self):
+        enum_table = self.tables['non_native_enum_table']
+
+        enum_table.insert().execute([
+            {'id': 1, 'someenum': 'two'},
+            {'id': 2, 'someenum': 'two'},
+            {'id': 3, 'someenum': 'one'},
+        ])
+
+        eq_(
+            select(['foo' + enum_table.c.someenum]).
+            order_by(enum_table.c.id).execute().fetchall(),
+            [
+                ('footwo', ),
+                ('footwo', ),
+                ('fooone', )
+            ]
+        )
 
     @testing.fails_on(
         'postgresql+zxjdbc',
         'zxjdbc fails on ENUM: column "XXX" is of type XXX '
         'but expression is of type character varying')
     def test_round_trip(self):
+        enum_table = self.tables['enum_table']
+
         enum_table.insert().execute([
             {'id': 1, 'someenum': 'two'},
             {'id': 2, 'someenum': 'two'},
@@ -1053,7 +1351,102 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
             ]
         )
 
+    def test_null_round_trip(self):
+        enum_table = self.tables.enum_table
+        non_native_enum_table = self.tables.non_native_enum_table
+
+        with testing.db.connect() as conn:
+            conn.execute(enum_table.insert(), {"id": 1, "someenum": None})
+            eq_(conn.scalar(select([enum_table.c.someenum])), None)
+
+        with testing.db.connect() as conn:
+            conn.execute(
+                non_native_enum_table.insert(), {"id": 1, "someenum": None})
+            eq_(conn.scalar(select([non_native_enum_table.c.someenum])), None)
+
+    @testing.requires.enforces_check_constraints
+    def test_check_constraint(self):
+        assert_raises(
+            (exc.IntegrityError, exc.ProgrammingError),
+            testing.db.execute,
+            "insert into non_native_enum_table "
+            "(id, someenum) values(1, 'four')")
+
+    @testing.requires.enforces_check_constraints
+    @testing.provide_metadata
+    def test_variant_we_are_default(self):
+        # test that the "variant" does not create a constraint
+        t = Table(
+            'my_table', self.metadata,
+            Column(
+                'data', Enum("one", "two", "three", name="e1").with_variant(
+                    Enum("four", "five", "six", name="e2"), "some_other_db"
+                )
+            )
+        )
+
+        eq_(
+            len([c for c in t.constraints if isinstance(c, CheckConstraint)]),
+            2
+        )
+
+        with testing.db.connect() as conn:
+            self.metadata.create_all(conn)
+            assert_raises(
+                (exc.IntegrityError, exc.ProgrammingError, exc.DataError),
+                conn.execute,
+                "insert into my_table "
+                "(data) values('four')")
+            conn.execute("insert into my_table (data) values ('two')")
+
+    @testing.requires.enforces_check_constraints
+    @testing.provide_metadata
+    def test_variant_we_are_not_default(self):
+        # test that the "variant" does not create a constraint
+        t = Table(
+            'my_table', self.metadata,
+            Column(
+                'data', Enum("one", "two", "three", name="e1").with_variant(
+                    Enum("four", "five", "six", name="e2"),
+                    testing.db.dialect.name
+                )
+            )
+        )
+
+        # ensure Variant isn't exploding the constraints
+        eq_(
+            len([c for c in t.constraints if isinstance(c, CheckConstraint)]),
+            2
+        )
+
+        with testing.db.connect() as conn:
+            self.metadata.create_all(conn)
+            assert_raises(
+                (exc.IntegrityError, exc.ProgrammingError, exc.DataError),
+                conn.execute,
+                "insert into my_table "
+                "(data) values('two')")
+            conn.execute("insert into my_table (data) values ('four')")
+
+    def test_skip_check_constraint(self):
+        with testing.db.connect() as conn:
+            conn.execute(
+                "insert into non_native_enum_table "
+                "(id, someotherenum) values(1, 'four')"
+            )
+            eq_(
+                conn.scalar("select someotherenum from non_native_enum_table"),
+                "four")
+            assert_raises_message(
+                LookupError,
+                '"four" is not among the defined enum values',
+                conn.scalar,
+                select([self.tables.non_native_enum_table.c.someotherenum])
+            )
+
     def test_non_native_round_trip(self):
+        non_native_enum_table = self.tables['non_native_enum_table']
+
         non_native_enum_table.insert().execute([
             {'id': 1, 'someenum': 'two'},
             {'id': 2, 'someenum': 'two'},
@@ -1061,7 +1454,9 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
         ])
 
         eq_(
-            non_native_enum_table.select().
+            select([
+                non_native_enum_table.c.id,
+                non_native_enum_table.c.someenum]).
             order_by(non_native_enum_table.c.id).execute().fetchall(),
             [
                 (1, 'two'),
@@ -1070,6 +1465,25 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
             ]
         )
 
+    def test_pep435_enum_round_trip(self):
+        stdlib_enum_table = self.tables['stdlib_enum_table']
+
+        stdlib_enum_table.insert().execute([
+            {'id': 1, 'someenum': self.SomeEnum.two},
+            {'id': 2, 'someenum': self.SomeEnum.two},
+            {'id': 3, 'someenum': self.SomeEnum.one},
+        ])
+
+        eq_(
+            stdlib_enum_table.select().
+            order_by(stdlib_enum_table.c.id).execute().fetchall(),
+            [
+                (1, self.SomeEnum.two),
+                (2, self.SomeEnum.two),
+                (3, self.SomeEnum.one),
+            ]
+        )
+
     def test_adapt(self):
         from sqlalchemy.dialects.postgresql import ENUM
         e1 = Enum('one', 'two', 'three', native_enum=False)
@@ -1079,6 +1493,9 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
         e1 = Enum('one', 'two', 'three', name='foo', schema='bar')
         eq_(e1.adapt(ENUM).name, 'foo')
         eq_(e1.adapt(ENUM).schema, 'bar')
+        e1 = Enum(self.SomeEnum)
+        eq_(e1.adapt(ENUM).name, 'someenum')
+        eq_(e1.adapt(ENUM).enums, ['one', 'two', 'three'])
 
     @testing.provide_metadata
     def test_create_metadata_bound_no_crash(self):
@@ -1087,29 +1504,19 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
 
         m1.create_all(testing.db)
 
-    @testing.crashes(
-        'mysql', 'Inconsistent behavior across various OS/drivers')
-    def test_constraint(self):
-        assert_raises(
-            exc.DBAPIError, enum_table.insert().execute,
-            {'id': 4, 'someenum': 'four'})
-
     def test_non_native_constraint_custom_type(self):
         class Foob(object):
 
             def __init__(self, name):
                 self.name = name
 
-        class MyEnum(types.SchemaType, TypeDecorator):
+        class MyEnum(TypeDecorator):
 
             def __init__(self, values):
                 self.impl = Enum(
                     *[v.name for v in values], name="myenum",
                     native_enum=False)
 
-            def _set_table(self, table, column):
-                self.impl._set_table(table, column)
-
             # future method
             def process_literal_param(self, value, dialect):
                 return value.name
@@ -1128,13 +1535,11 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
             dialect="default"
         )
 
-    @testing.fails_on(
-        'mysql',
-        "the CHECK constraint doesn't raise an exception for unknown reason")
-    def test_non_native_constraint(self):
+    def test_lookup_failure(self):
         assert_raises(
-            exc.DBAPIError, non_native_enum_table.insert().execute,
-            {'id': 4, 'someenum': 'four'}
+            exc.StatementError,
+            self.tables['non_native_enum_table'].insert().execute,
+            {'id': 4, 'someotherenum': 'four'}
         )
 
     def test_mock_engine_no_prob(self):
@@ -1158,6 +1563,7 @@ class EnumTest(AssertsCompiledSQL, fixtures.TestBase):
             "Enum('x', 'y', name='somename', "
             "inherit_schema=True, native_enum=False)")
 
+
 binary_table = MyPickleType = metadata = None
 
 
@@ -1228,14 +1634,14 @@ class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
                     'data': LargeBinary, 'data_slice': LargeBinary},
                 bind=testing.db)
         ):
-            l = stmt.execute().fetchall()
-            eq_(stream1, l[0]['data'])
-            eq_(stream1[0:100], l[0]['data_slice'])
-            eq_(stream2, l[1]['data'])
-            eq_(testobj1, l[0]['pickled'])
-            eq_(testobj2, l[1]['pickled'])
-            eq_(testobj3.moredata, l[0]['mypickle'].moredata)
-            eq_(l[0]['mypickle'].stuff, 'this is the right stuff')
+            result = stmt.execute().fetchall()
+            eq_(stream1, result[0]['data'])
+            eq_(stream1[0:100], result[0]['data_slice'])
+            eq_(stream2, result[1]['data'])
+            eq_(testobj1, result[0]['pickled'])
+            eq_(testobj2, result[1]['pickled'])
+            eq_(testobj3.moredata, result[0]['mypickle'].moredata)
+            eq_(result[0]['mypickle'].stuff, 'this is the right stuff')
 
     @testing.requires.binary_comparisons
     def test_comparison(self):
@@ -1266,6 +1672,215 @@ class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
         with open(f, mode='rb') as o:
             return o.read()
 
+
+class JSONTest(fixtures.TestBase):
+
+    def setup(self):
+        metadata = MetaData()
+        self.test_table = Table('test_table', metadata,
+                                Column('id', Integer, primary_key=True),
+                                Column('test_column', JSON),
+                                )
+        self.jsoncol = self.test_table.c.test_column
+
+        self.dialect = default.DefaultDialect()
+        self.dialect._json_serializer = None
+        self.dialect._json_deserializer = None
+
+    def test_bind_serialize_default(self):
+        proc = self.test_table.c.test_column.type._cached_bind_processor(
+            self.dialect)
+        eq_(
+            proc({"A": [1, 2, 3, True, False]}),
+            '{"A": [1, 2, 3, true, false]}'
+        )
+
+    def test_bind_serialize_None(self):
+        proc = self.test_table.c.test_column.type._cached_bind_processor(
+            self.dialect)
+        eq_(
+            proc(None),
+            'null'
+        )
+
+    def test_bind_serialize_none_as_null(self):
+        proc = JSON(none_as_null=True)._cached_bind_processor(
+            self.dialect)
+        eq_(
+            proc(None),
+            None
+        )
+        eq_(
+            proc(null()),
+            None
+        )
+
+    def test_bind_serialize_null(self):
+        proc = self.test_table.c.test_column.type._cached_bind_processor(
+            self.dialect)
+        eq_(
+            proc(null()),
+            None
+        )
+
+    def test_result_deserialize_default(self):
+        proc = self.test_table.c.test_column.type._cached_result_processor(
+            self.dialect, None)
+        eq_(
+            proc('{"A": [1, 2, 3, true, false]}'),
+            {"A": [1, 2, 3, True, False]}
+        )
+
+    def test_result_deserialize_null(self):
+        proc = self.test_table.c.test_column.type._cached_result_processor(
+            self.dialect, None)
+        eq_(
+            proc('null'),
+            None
+        )
+
+    def test_result_deserialize_None(self):
+        proc = self.test_table.c.test_column.type._cached_result_processor(
+            self.dialect, None)
+        eq_(
+            proc(None),
+            None
+        )
+
+    def _dialect_index_fixture(self, int_processor, str_processor):
+        class MyInt(Integer):
+            def bind_processor(self, dialect):
+                return lambda value: value + 10
+
+            def literal_processor(self, diaect):
+                return lambda value: str(value + 15)
+
+        class MyString(String):
+            def bind_processor(self, dialect):
+                return lambda value: value + "10"
+
+            def literal_processor(self, diaect):
+                return lambda value: value + "15"
+
+        class MyDialect(default.DefaultDialect):
+            colspecs = {}
+            if int_processor:
+                colspecs[Integer] = MyInt
+            if str_processor:
+                colspecs[String] = MyString
+
+        return MyDialect()
+
+    def test_index_bind_proc_int(self):
+        expr = self.test_table.c.test_column[5]
+
+        int_dialect = self._dialect_index_fixture(True, True)
+        non_int_dialect = self._dialect_index_fixture(False, True)
+
+        bindproc = expr.right.type._cached_bind_processor(int_dialect)
+        eq_(bindproc(expr.right.value), 15)
+
+        bindproc = expr.right.type._cached_bind_processor(non_int_dialect)
+        eq_(bindproc(expr.right.value), 5)
+
+    def test_index_literal_proc_int(self):
+        expr = self.test_table.c.test_column[5]
+
+        int_dialect = self._dialect_index_fixture(True, True)
+        non_int_dialect = self._dialect_index_fixture(False, True)
+
+        bindproc = expr.right.type._cached_literal_processor(int_dialect)
+        eq_(bindproc(expr.right.value), "20")
+
+        bindproc = expr.right.type._cached_literal_processor(non_int_dialect)
+        eq_(bindproc(expr.right.value), "5")
+
+    def test_index_bind_proc_str(self):
+        expr = self.test_table.c.test_column['five']
+
+        str_dialect = self._dialect_index_fixture(True, True)
+        non_str_dialect = self._dialect_index_fixture(False, False)
+
+        bindproc = expr.right.type._cached_bind_processor(str_dialect)
+        eq_(bindproc(expr.right.value), 'five10')
+
+        bindproc = expr.right.type._cached_bind_processor(non_str_dialect)
+        eq_(bindproc(expr.right.value), 'five')
+
+    def test_index_literal_proc_str(self):
+        expr = self.test_table.c.test_column['five']
+
+        str_dialect = self._dialect_index_fixture(True, True)
+        non_str_dialect = self._dialect_index_fixture(False, False)
+
+        bindproc = expr.right.type._cached_literal_processor(str_dialect)
+        eq_(bindproc(expr.right.value), "five15")
+
+        bindproc = expr.right.type._cached_literal_processor(non_str_dialect)
+        eq_(bindproc(expr.right.value), "'five'")
+
+
+class ArrayTest(fixtures.TestBase):
+
+    def _myarray_fixture(self):
+        class MyArray(ARRAY):
+            pass
+        return MyArray
+
+    def test_array_index_map_dimensions(self):
+        col = column('x', ARRAY(Integer, dimensions=3))
+        is_(
+            col[5].type._type_affinity, ARRAY
+        )
+        eq_(
+            col[5].type.dimensions, 2
+        )
+        is_(
+            col[5][6].type._type_affinity, ARRAY
+        )
+        eq_(
+            col[5][6].type.dimensions, 1
+        )
+        is_(
+            col[5][6][7].type._type_affinity, Integer
+        )
+
+    def test_array_getitem_single_type(self):
+        m = MetaData()
+        arrtable = Table(
+            'arrtable', m,
+            Column('intarr', ARRAY(Integer)),
+            Column('strarr', ARRAY(String)),
+        )
+        is_(arrtable.c.intarr[1].type._type_affinity, Integer)
+        is_(arrtable.c.strarr[1].type._type_affinity, String)
+
+    def test_array_getitem_slice_type(self):
+        m = MetaData()
+        arrtable = Table(
+            'arrtable', m,
+            Column('intarr', ARRAY(Integer)),
+            Column('strarr', ARRAY(String)),
+        )
+        is_(arrtable.c.intarr[1:3].type._type_affinity, ARRAY)
+        is_(arrtable.c.strarr[1:3].type._type_affinity, ARRAY)
+
+    def test_array_getitem_slice_type_dialect_level(self):
+        MyArray = self._myarray_fixture()
+        m = MetaData()
+        arrtable = Table(
+            'arrtable', m,
+            Column('intarr', MyArray(Integer)),
+            Column('strarr', MyArray(String)),
+        )
+        is_(arrtable.c.intarr[1:3].type._type_affinity, ARRAY)
+        is_(arrtable.c.strarr[1:3].type._type_affinity, ARRAY)
+
+        # but the slice returns the actual type
+        assert isinstance(arrtable.c.intarr[1:3].type, MyArray)
+        assert isinstance(arrtable.c.strarr[1:3].type, MyArray)
+
+
 test_table = meta = MyCustomType = MyTypeDec = None
 
 
@@ -1477,12 +2092,52 @@ class ExpressionTest(
             use_default_dialect=True
         )
 
-        assert expr.type.__class__ is MyTypeDec
+        is_(expr.right.type.__class__, MyTypeDec)
+        is_(expr.type.__class__, MyTypeDec)
+
         eq_(
             testing.db.execute(select([expr.label('foo')])).scalar(),
             "BIND_INfooBIND_IN6BIND_OUT"
         )
 
+    def test_variant_righthand_coercion_honors_wrapped(self):
+        my_json_normal = JSON()
+        my_json_variant = JSON().with_variant(String(), "sqlite")
+
+        tab = table(
+            'test',
+            column('avalue', my_json_normal),
+            column('bvalue', my_json_variant)
+        )
+        expr = tab.c.avalue['foo'] == 'bar'
+
+        is_(expr.right.type._type_affinity, String)
+        is_not_(expr.right.type, my_json_normal)
+
+        expr = tab.c.bvalue['foo'] == 'bar'
+
+        is_(expr.right.type._type_affinity, String)
+        is_not_(expr.right.type, my_json_variant)
+
+    def test_variant_righthand_coercion_returns_self(self):
+        my_datetime_normal = DateTime()
+        my_datetime_variant = DateTime().with_variant(
+            dialects.sqlite.DATETIME(truncate_microseconds=False), "sqlite")
+
+        tab = table(
+            'test',
+            column('avalue', my_datetime_normal),
+            column('bvalue', my_datetime_variant)
+        )
+        expr = tab.c.avalue == datetime.datetime(2015, 10, 14, 15, 17, 18)
+
+        is_(expr.right.type._type_affinity, DateTime)
+        is_(expr.right.type, my_datetime_normal)
+
+        expr = tab.c.bvalue == datetime.datetime(2015, 10, 14, 15, 17, 18)
+
+        is_(expr.right.type, my_datetime_variant)
+
     def test_bind_typing(self):
         from sqlalchemy.sql import column
 
@@ -1500,7 +2155,7 @@ class ExpressionTest(
         # untyped bind - it gets assigned MyFoobarType
         bp = bindparam("foo")
         expr = column("foo", MyFoobarType) + bp
-        assert bp.type._type_affinity is types.NullType
+        assert bp.type._type_affinity is types.NullType  # noqa
         assert expr.right.type._type_affinity is MyFoobarType
 
         expr = column("foo", MyFoobarType) + bindparam("foo", type_=Integer)
@@ -1571,6 +2226,33 @@ class ExpressionTest(
         assert distinct(test_table.c.data).type == test_table.c.data.type
         assert test_table.c.data.distinct().type == test_table.c.data.type
 
+    def test_detect_coercion_of_builtins(self):
+        @inspection._self_inspects
+        class SomeSQLAThing(object):
+            def __repr__(self):
+                return "some_sqla_thing()"
+
+        class SomeOtherThing(object):
+            pass
+
+        assert_raises_message(
+            exc.ArgumentError,
+            r"Object some_sqla_thing\(\) is not legal as a SQL literal value",
+            lambda: column('a', String) == SomeSQLAThing()
+        )
+
+        is_(
+            bindparam('x', SomeOtherThing()).type,
+            types.NULLTYPE
+        )
+
+    def test_detect_coercion_not_fooled_by_mock(self):
+        m1 = mock.Mock()
+        is_(
+            bindparam('x', m1).type,
+            types.NULLTYPE
+        )
+
 
 class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = 'default'
@@ -1732,6 +2414,7 @@ class NumericRawSQLTest(fixtures.TestBase):
         else:
             eq_(val, 46.583)
 
+
 interval_table = metadata = None
 
 
@@ -1839,12 +2522,9 @@ class BooleanTest(
             def __init__(self, value):
                 self.value = value
 
-        class MyBool(types.SchemaType, TypeDecorator):
+        class MyBool(TypeDecorator):
             impl = Boolean()
 
-            def _set_table(self, table, column):
-                self.impl._set_table(table, column)
-
             # future method
             def process_literal_param(self, value, dialect):
                 return value.value
@@ -1863,6 +2543,39 @@ class BooleanTest(
             dialect="sqlite"
         )
 
+    @testing.skip_if(lambda: testing.db.dialect.supports_native_boolean)
+    def test_nonnative_processor_coerces_to_onezero(self):
+        boolean_table = self.tables.boolean_table
+        with testing.db.connect() as conn:
+            conn.execute(
+                boolean_table.insert(),
+                {"id": 1, "unconstrained_value": 5}
+            )
+
+            eq_(
+                conn.scalar("select unconstrained_value from boolean_table"),
+                1
+            )
+
+    @testing.skip_if(lambda: testing.db.dialect.supports_native_boolean)
+    def test_nonnative_processor_coerces_integer_to_boolean(self):
+        boolean_table = self.tables.boolean_table
+        with testing.db.connect() as conn:
+            conn.execute(
+                "insert into boolean_table (id, unconstrained_value) "
+                "values (1, 5)"
+            )
+
+            eq_(
+                conn.scalar("select unconstrained_value from boolean_table"),
+                5
+            )
+
+            eq_(
+                conn.scalar(select([boolean_table.c.unconstrained_value])),
+                True
+            )
+
 
 class PickleTest(fixtures.TestBase):
 
@@ -1891,6 +2604,7 @@ class PickleTest(fixtures.TestBase):
         ):
             assert p1.compare_values(p1.copy_value(obj), obj)
 
+
 meta = None
 
 
