@@ -51,7 +51,7 @@ class _UpdateFromTestBase(object):
                 (9, 'fred'),
                 (10, 'chuck')
             ),
-            addresses = (
+            addresses=(
                 ('id', 'user_id', 'name', 'email_address'),
                 (1, 7, 'x', 'jack@bean.com'),
                 (2, 8, 'x', 'ed@wood.com'),
@@ -59,7 +59,7 @@ class _UpdateFromTestBase(object):
                 (4, 8, 'x', 'ed@lala.com'),
                 (5, 9, 'x', 'fred@fred.com')
             ),
-            dingalings = (
+            dingalings=(
                 ('id', 'address_id', 'data'),
                 (1, 2, 'ding 1/2'),
                 (2, 5, 'ding 2/5')
@@ -306,8 +306,8 @@ class UpdateTest(_UpdateFromTestBase, fixtures.TablesTest, AssertsCompiledSQL):
         table1 = self.tables.mytable
         testing.assert_raises_message(
             ValueError,
-            "When preserve_parameter_order is True, values\(\) "
-            "only accepts a list of 2-tuples",
+            r"When preserve_parameter_order is True, values\(\) "
+            r"only accepts a list of 2-tuples",
             table1.update(preserve_parameter_order=True).values,
             {"description": "foo", "name": "bar"}
         )
@@ -340,13 +340,14 @@ class UpdateTest(_UpdateFromTestBase, fixtures.TablesTest, AssertsCompiledSQL):
     def test_where_empty(self):
         table1 = self.tables.mytable
         self.assert_compile(
-            table1.update().where(
-                and_()),
-            "UPDATE mytable SET myid=:myid, name=:name, description=:description")
+            table1.update().where(and_()),
+            "UPDATE mytable SET myid=:myid, name=:name, "
+            "description=:description")
         self.assert_compile(
             table1.update().where(
                 or_()),
-            "UPDATE mytable SET myid=:myid, name=:name, description=:description")
+            "UPDATE mytable SET myid=:myid, name=:name, "
+            "description=:description")
 
     def test_prefix_with(self):
         table1 = self.tables.mytable
@@ -369,7 +370,7 @@ class UpdateTest(_UpdateFromTestBase, fixtures.TablesTest, AssertsCompiledSQL):
 
         this logic is triggered currently by a left side that doesn't
         have a key.  The current supported use case is updating the index
-        of a Postgresql ARRAY type.
+        of a PostgreSQL ARRAY type.
 
         """
         table1 = self.tables.mytable
